#ident	"$Revision: 3.17 $"

/*
 * Copyright 1985 by MIPS Computer Systems, Inc.
 */


/**************************************************************************
 *									  *
 * 		 Copyright (C) 1988, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

/*
 * dkipreg.h -- register definitions for Interphase VME 3200 SMD controller
 *		define NOMACSI to disable the 4201/4200 from using the MACSI
 *		interface. MACSI allows command Queing, sorting, overlapped
 *		seeks and will automatically clear drive faults.
 */

#define BIT(x)	(0x01 << x)

/*
 * Scatter gather structure.  Using the scatter gather structure, the host
 * can make virtually contiguous dma appear physically contiguous to the
 * controller.  The controller can handle a maximum of a sector size,
 * in bytes, of scatter gather entries.  All the entries must use the
 * same address modifier and memory space.
 */
typedef struct ipsg {
	ushort	sg_count;		/* byte/entry count */
	ushort	sg_addrhi;		/* datablock/entry address high */
	ushort	sg_addrlo;		/* datablock/entry address low */
	unchar	sg_meminfo;		/* bits about memory location */
	unchar	sg_addrmod;		/* VME address modifier */
}IPSG;

/*
 * format of Interphase VME 3200 i/o parameter block
 * NOTE: assumed to be accessed via a D16 data path
 */
typedef struct ipiopb {
	unchar	ip_cmdcode;		/* 00: command code */
	unchar	ip_cmdopt;		/* 01: command options */
	unchar	ip_statcode;		/* 02: status code */
	unchar	ip_errcode;		/* 03: error code */
	union {
		struct {
			ushort	idp_cyl;	/* 04,05: cylinder */
			unchar	idp_head;	/* 06: head */
			unchar	idp_sec;	/* 07: sector */
		} id_pda;			/* physical disk address */
		struct {
			ushort idl_lbnhi;	/* 04,05: logical block high */
			ushort idl_lbnlo;	/* 06,07: logical block low */
		} id_lda;			/* logical disk address */
	} ip_diskaddr;		/* disk address */
	ushort	ip_scnt;		/* 08,09: sector count */
	ushort	ip_bahi;		/* 0A,0B: buffer address high */
	ushort	ip_balo;		/* 0C,0D: buffer address low */
	unchar	ip_memtype;		/* 0E: memory type */
	unchar	ip_addrmod;		/* 0F: VME address modifier */
	unchar	ip_ipl;			/* 10: interrupt priority level */
	unchar	ip_normvec;		/* 11: normal completion intr vector */
	unchar	ip_dmaburst;		/* 12: dma burst length */
	unchar	ip_errvec;		/* 13: error completion intr vector */
	ushort	ip_iopbhi;		/* 14,15: next iopb address high */
	ushort	ip_iopblo;		/* 16,17: next iopb address low */
	unchar	ip_pbmemtype;		/* 18: iopb memory type */
	unchar	ip_pbaddrmod;		/* 19: iopb VME address modifier */
	unchar	ip_skew;		/* 1A: spiral skew */
	unchar	ip_sgentries;		/* 1B: number of scatter/gather entries 
					 * in the first scatter/gather list */
}IPIOPB;

/*
 * shorthand for disk address
 */
#define	ip_lbnhi	ip_diskaddr.id_lda.idl_lbnhi
#define	ip_lbnlo	ip_diskaddr.id_lda.idl_lbnlo

#define	ip_cyl	ip_diskaddr.id_pda.idp_cyl
#define	ip_head	ip_diskaddr.id_pda.idp_head
#define	ip_sec	ip_diskaddr.id_pda.idp_sec

/*
 * command codes
 * (ip_cmdcode)
 */
#define	IP_DIAG		0x70		/* run power-up diagnostics */
#define	IP_READLONG	0x71		/* read data and ecc */
#define	IP_WRITELONG	0x72		/* write data and ecc */
#define	IP_READHDR	0x74		/* read first encountered hdr */
#define IP_READRAW	0x75		/* read raw data */
#define IP_READFLAWS	0x76		/* read cdc flaw map */
#define	IP_GETCONFIG	0x77		/* get uib configuration */
#define	IP_WRITESBUF	0x78		/* write sector buffer */
#define	IP_READSBUF	0x79		/* read sector buffer */
#define IP_GMACSI	0x7f		/* put controller in MACSI mode	*/
#define	IP_READ		0x81		/* read sectors */
#define	IP_WRITE	0x82		/* write sectors */
#define	IP_VERIFY	0x83		/* verify sectors */
#define	IP_FORMATTRK	0x84		/* format track */
#define	IP_MAPTRK	0x85		/* forward track */
#define	IP_HANDSHAKE	0x86		/* get firmware id */
#define	IP_INITIALIZE	0x87		/* set uib configuration */
#define	IP_RECAL	0x89		/* rtz and recalibrate drive */
#define	IP_SEEK		0x8A		/* seek to cylinder */
#define	IP_REFORMAT	0x8B		/* format preserving track forwards */
#define	IP_FMTDATA	0x8C		/* format and write data */
#define	IP_MAPSEC	0x90		/* forward sector */
#define IP_READSEQ	0x91		/* read sectors sequential */
#define IP_WRITESEQ	0x92		/* write sectors sequential */
#define IP_VERIFYSEQ	0x93		/* verify sectors sequential */
#define	IP_READNC	0x94		/* read bypassing controller cache */
#define IP_READSEQDA	0x95		/* read sequential, disable addr */
#define IP_WRITESEQDA	0x96		/* write sequential, disable addr */
#define IP_CLRFAULT	0x97		/* clear drive fault */
#define	IP_VERIFYTRK	0x99		/* verify trk readable w/o ecc errs */
#define	IP_TRACKID	0x9A		/* return sec hdrs in order encntrd */
#define	IP_EXECIOPB	0x9B		/* execute linked iopb's */
#define IP_VERIFYTRKSEQ	0x9C		/* verify track sequential */
#define IP_EDIAGS	0x9E		/* extended diagnostics */
#define IP_DPPRISEL	0x9F		/* dual port priority select */
#define IP_SCATTER	0xA1		/* read sectors & scatter to memory */
#define IP_GATHER	0xA2		/* gather from memory & write sectors */
#define IP_WSCATTER	0xA3		/* word wide scatter (no zero latency)*/
#define IP_WGATHER	0xA4		/* word wide gather (no zero latency)*/

/*
 * command options
 * (ip_cmdopt)
 */
#define	IPO_ECCEN	0x01		/* enable ecc correction */
#define	IPO_INTEN	0x02		/* enable cmd completion interrupt */
#define	IPO_DISERR	0x04		/* disable ecc error detection */
#define	IPO_RSVPORT	0x08		/* reserve dual ported drive */
#define	IPO_LOGICAL	0x10		/* enable logical block addressing */
#define	IPO_LINKPG	0x20		/* another iopb is linked */
#define	IPO_VOLUME	0x40		/* volume number */
#define	IPO_UNIT	0x80		/* unit number */

/*
 * command status codes
 * (ip_statcode)
 */
#define	IPS_OK		0x80		/* completed without errors */
#define	IPS_INPROG	0x81		/* in progress */
#define	IPS_ERROR	0x82		/* completed with error */
#define	IPS_EXCEPT	0x83		/* completed with exception */

/*
 * command exception codes (valid if ip_statcode == IPS_EXCEPT)
 * (ip_errcode)
 */
#define	IPEX_RETRYMASK	0x0F		/* mask for retry count */
#define	IPEX_RECAL	0x10		/* recalibrate done */
#define	IPEX_BADDATA	0x40		/* data may be bad */
#define	IPEX_ECC	0x80		/* ecc correction done */
#define	IPEX_ICKY	(IPEX_ECC|IPEX_BADDATA|IPEX_RECAL)

/*
 * command error code (valid if ip_statcode == IPS_ERROR)
 * (ip_errcode)
 */
#define	IPER_DNOTRDY	0x10		/* drive not ready */
#define	IPER_SEEKERR	0x12		/* seek error, cyl not found */
#define	IPER_ECCERR	0x13		/* ecc error, correctn not attempted */
#define	IPER_INVCMD	0x14		/* invalid command */
#define	IPER_ILLEXEC	0x15		/* illegal execute iopb */
#define	IPER_INVSEC	0x16		/* invalid sector address */
#define	IPER_ILLMEM	0x17		/* illegal memory type */
#define	IPER_BUSTMOUT	0x18		/* bus timeout */
#define	IPER_HDRCSUM	0x19		/* header checksum */
#define	IPER_WPROT	0x1A		/* write protect */
#define	IPER_NOUNIT	0x1B		/* unit select error */
#define	IPER_SKERRTMOUT	0x1C		/* seek error timeout */
#define	IPER_FAULTTMOUT	0x1D		/* fault timeout */
#define	IPER_DFAULT	0x1E		/* drive fault */
#define	IPER_RDYTMOUT	0x1F		/* ready timeout */
#define	IPER_EOM	0x20		/* end of media */
#define	IPER_VOLFAULT	0x21		/* volume uib not initialized */
#define	IPER_INVHDRPAD	0x22		/* invalid header pad */
#define	IPER_HARDECC	0x23		/* uncorrectable ecc error */
#define	IPER_LGCLCYL	0x24		/* logical address error on cylinder */
#define	IPER_LGCLTRK	0x25		/* logical address error on trk */
#define	IPER_LGCLSEC	0x26		/* logical address error on sec */
#define	IPER_DOVERRUN	0x27		/* data overrun */
#define	IPER_INDEXTMOUT	0x28		/* index pulse timeout */
#define	IPER_SECNFND	0x29		/* sector not found */
#define	IPER_IDERR	0x2A		/* head number wrong in header */
#define	IPER_INVSYNCD	0x2B		/* invalid sync char in data */
#define IPER_HDRINVAL	0x2C		/* no valid header was found */
#define	IPER_SEEKTMOUT	0x2D		/* seek timeout */
#define	IPER_BUSYTMOUT	0x2E		/* busy timeout */
#define	IPER_NOTONCYL	0x2F		/* not on cylinder */
#define	IPER_RTZTMOUT	0x30		/* rtz timeout */
#define	IPER_INVSYNCH	0x31		/* invalid sync char in header */
#define	IPER_UNITINIT	0x40		/* unit not initialized */
#define	IPER_GAPERR	0x42		/* gap specification error */
#define	IPER_DSEEKERR	0x4B		/* drive reported seek error */
#define	IPER_UIBSECTRK	0x50		/* uib sectors/trk spec incorrect */
#define	IPER_UIBBYTSEC	0x51		/* uib bytes/sector spec incorrect */
#define	IPER_UIBINTRLV	0x52		/* uib intrlv spec incorrect */
#define	IPER_INVTRK	0x53		/* trk address mismatch with uib */
#define	IPER_INVCYL	0x54		/* cyl address mismatch with uib */
#define	IPER_ODDDMACNT	0x5D		/* odd dma count */
#define	IPER_PBBUSERR	0x60		/* bus error referencing iopb */
#define	IPER_DMABUSERR	0x61		/* bus error on data transfer */
#define	IPER_ADDRERR	0x62		/* unaligned address */
#define IPER_BADHDR	0x6A		/* unrecognized header field */
#define IPER_BADMAPHDR	0x6B		/* mapped header error */
#define	IPER_NOSPARE	0x6F		/* spare sector not spec'ed in uib */
#define IPER_CMDABORT	0x77		/* command aborted */
#define IPER_ACFAIL	0x78		/* acfail detected */
#define	IPER_CMDNIMP	0xFF		/* command not implemented */

/*
 * memory types
 * (ip_memtype and ip_pbmemtype)
 */
#define	IPMT_8BIT	0		/* 8 bit data transfers */
#define	IPMT_16BITI	1		/* 16 bit internal memory */
#define	IPMT_16BIT	2		/* 16 bit data transfers */
#define	IPMT_32BIT	3		/* 32 bit data transfers */

/*
 * format of Interphase VME 3200 device registers
 * NOTE: this is accessed via a D16 (not D32!) data path
 * For normal transfers, an iopb in this io space is used, as well as its
 * scatter gather entries.  If the transfer can't take place under such
 * restrictions, then the scatter gather info is stored in host memory
 * instead.
 */
#define IP_MAXSG	59		/* max # of sg entries that fit */
typedef struct ipdevice {
	unchar	ipds1;			/* 000: drive status unit 1 */
	unchar	ipds0;			/* 001: drive status unit 0 */
	ushort	ipcsr;			/* 002,003: command/status register */
	struct	ipiopb ipiopb;		/* 004-023: iopb */
	struct	ipsg ipsg[IP_MAXSG];	/* 024-1F7: some sg entries */
	char	ipreserved[2];		/* 1F8-1F9: not used */
	/*
	 * The remaining are only defined for the 3201, and then only if
	 * the four drive adapter is in place.  In any case, on the 3201
	 * you can't use these i/o registers for anything else.
	 */
#ifdef OLD_WAY
	unchar	ipds3;			/* 1FA: drive status unit 3 */
	unchar	ipds2;			/* 1FB: drive status unit 2 */
	unchar	ipds1a;			/* 1FC: drive status unit 1 */
	unchar	ipds0a;			/* 1FD: drive status unit 0 */
#endif
	unchar	ipds4[4];		/* 4 unit drive statuses	*/
	ushort	ipdscr;			/* 1FE,1FF: status change register */
}IPDEVICE;

/*
 * drive status bits
 * (ipds0 and ipds1)
 */
#define	IPDS_DRDY	0x01	/* drive ready */
#define	IPDS_WPROT	0x02	/* drive write protected */
#define	IPDS_DBUSY	0x04	/* drive busy, dual ported drives only */
#define	IPDS_FAULT	0x08	/* drive fault */
#define	IPDS_ONCYL	0x10	/* drive on cylinder */
#define	IPDS_SKERR	0x20	/* drive seek error */
#define	IPDS_UALIVE	0x40	/* controller has detected drive */
#define	IPDS_URDY	0x80	/* ctrlr sees DRDY & ONCYL & !FAULT & !SKERR */

/*
 * command and status bits
 * (ipcsr)
 */
#define	IPCS_SCSRC	0x0008	/* status change source, drv 0 or 1 */
#define	IPCS_ERLST	0x0010	/* error detected on last command */
#define	IPCS_SC		0x0020	/* status change (DRDY,FAULT,ONCYL,SKERR) */
#define	IPCS_OPDONE	0x0040	/* operation done */
#define	IPCS_GOBUSY	0x0080	/* go/busy bit */
#define	IPCS_BERR	0x0100	/* bus error detected */
#define IPCS_ABORT	0x0800	/* abort current operation */
#define	IPCS_BDCLR	0x1000	/* board clear */
#define	IPCS_SFEN	0x2000	/* VME sysfail enable */
#define	IPCS_BOK	0x4000	/* board ok */
#define	IPCS_SLED	0x8000	/* status led, 0=>show activity, 1=>green */

/*----------------------------------------------------------------------*/
/* NEW NON-MACSI structures						*
/*----------------------------------------------------------------------*/

/*
 * Product codes returned from the handshake command
 */
#define IP3201		0x65		/* Guerrila product code	*/
#define IP4201		0x74		/* Panther product code		*/
#define IP3200		0x49		/* 3200 SMD product code	*/
#define IP4200		0x66		/* 4200 SMD product code	*/
#define IP4400		0x82		/* Phoenix 4400 SMD product code */
#define IP4460		0x30		/* V/IPI 4460 IPI product code	*/

#ifdef MIPSEB
typedef struct
{
	unchar	p_code;
	unchar	p_var;
	unchar	p_revh;
	unchar	p_revl;
	unchar	p_idh;
	unchar	p_idl;
	unchar	p_month;	/* month stored as BCD	*/
	unchar	p_day;		/* day stored as BCD	*/
	short	p_year;		/* year stored  as BCD	*/
}HNDSHK;
#endif

#define PL_MSPARE	BIT(0)	/* supports multiple spares	*/
#define	PL_SOFT		BIT(1)	/* can run soft sector drives	*/
#define PL_DUAL		BIT(2)	/* supports dual porting	*/
#define PL_MACSI	BIT(3)	/* supports MACSI		*/

typedef struct
{
	unchar	pl_pcode;	/* product  code		*/
	char	*pl_pname;	/* product name			*/
	int	pl_flags;	/* poduct capabilities flags	*/
}PLIST;


/*----------------------------------------------------------------------*/
/* MACSI (Multiply Active Command System Interface) structures		*
/*----------------------------------------------------------------------*/
#ifndef NOMACSI
typedef ushort USHORT;
typedef unchar BYTE;
typedef unsigned int UINT;
#ifdef MIPSEB
typedef struct cqe	{			/* command queue entry	*/
			USHORT control;
			USHORT iopbaddr;	/* short i/o offset of iopb */
			USHORT cmdtag_h;
			USHORT cmdtag_l;
			BYTE length;		/* Lword size of iopb	*/
			BYTE queue;		/* unused for 4201/4200	*/
			USHORT rsrvd;
			}CQE;

typedef struct mcsb	{			/* Master status block	*/
			USHORT msr;
			USHORT mcr;
			USHORT iqar;
			USHORT qhead;		/* see note in define of */
						/* MACSI_CQE_INDX */
			USHORT rsrvd[4];
			}MCSB;

typedef struct crb      {			/* command response block */
			USHORT crsw;
			USHORT rsrvd;
			USHORT   cmdtag_h;
			USHORT   cmdtag_l;
			BYTE   length;
			BYTE   queue;
			USHORT rsrvd1;
			IPIOPB   copyiopb;
			}CRB;

typedef struct cib      {			/*MACSI initialization*/
			USHORT cib_num_cqes;
			USHORT cib_crb_off;
			USHORT cib_m_flags;	/* MACSI FLAGS	*/
			USHORT cib_num_free;	/* NUM free entries */
			USHORT cib_ce_vectr;
			USHORT cib_cn_vectr;
			USHORT cib_num_back;
			USHORT cib_r_robin;
			USHORT cib_res2;	/* Future use set to 0	*/
			USHORT cib_res3;	/* Future use set to 0	*/
			USHORT cib_res4;	/* Future use set to 0	*/
			USHORT cib_res5;	/* Future use set to 0	*/
			USHORT cib_res6;	/* Future use set to 0	*/
			USHORT cib_res7;	/* Future use set to 0	*/
			USHORT cib_res8;	/* Future use set to 0	*/
			USHORT cib_res9;	/* Future use set to 0	*/
			}VSMD_CIB;
#endif

/* MACSI mode constants */
#define NUM_CQES 	8		/* Command Q entries in short i/o */
#define NUM_MACSI	20		/* number of commands to queue */
#if defined(IP5) || defined(IP17)
#define NUM_M_SG 	5		/* number of SG tables per ctlr */
#define NUM_M_MAP	NUM_MACSI+1	/* number of dmamaps per ctlr */
#else
#define NUM_M_SG 	NUM_MACSI+1	/* number of SG tables per ctlr */
#endif
#define NUM_MCMD	NUM_MACSI+1	/* number of mcmd structs per ctlr */
#define MACSI_SG	64		/* number of SG entries per SG table */
#define NUM_R_ROBIN	8

/* When Running MACSI, Scatter/gather tables MUST be in external memory
 * We will declare a free pool of scatter/gather tables in each ctlr struct
 * NUM_M_SG defines the number of entries in the free pool per controller.
 * This number should be:
 *	NUM_MACSI to ensure that all the Q entries can be filled
 * However, The driver can deal with this being smaller.
 * NOTE:
 *	It is inavisable to make NUM_M_SG smaller than DKIPUPC.
 *	This will possibly prevent overlapped seeks from happening.
 *	2 * DKIPUPC is the smallest recommended number
 *
 *	The driver will add & remove from the top of the free list.
 * ALSO:
 *	The free list had to be in each ctlr struct instead of 1 common
 *	pool because if the controller start routine gets called at interrupt
 *	level, and there are no more free entries available, sleep cannot
 *	be called and there might not be any more requests comming in for
 *	that controller.
 */

typedef struct ipsg_free
{
	struct ipsg_free *nxt;		/* pointer to next IPSG_FREE	*/
	IPSG	ipsg[MACSI_SG];
} IPSG_FREE;

#if defined(IP5) || defined(IP17)
struct cpusg_free
{
	struct cpusg_free *next;
	dmamap_t	*dmamap;
};
#endif

#define IOPB_BUF_SIZ	sizeof(IPIOPB)

typedef struct
{
	char iopb_buf[IOPB_BUF_SIZ];
} IOPB_BUFF;

/*
 *	The sructure definition below is for the:
 *
 *		 VSMD, 3201, 4200 & 4201 short/io memory space
 *		IN MACSI mode
 *	NOTE: the CRB can actually be anywhere in short i/o 1a6 was just
 *	      picked out of the 'air'.
 */
#define U4STATBASE	(0x1fa)		/* address of 4 drive status	*/
#define CRB_OFFSET	(0x1a6)		/* address of Command RES. block */
#define CSLOP		(CRB_OFFSET - sizeof(MCSB) - (NUM_CQES * (sizeof(CQE) + sizeof(IOPB_BUFF))) - sizeof(VSMD_CIB))
#define MSLOP	(U4STATBASE  - CRB_OFFSET - sizeof(CRB))
/*
 * SHORT i/o definition for 4200/4201 in MACSI mode
 *
 */

typedef struct {
	MCSB		vsmd_mcsb;		/* this cannot move */
	CQE		vsmd_cqe[NUM_CQES];	/* this cannot move */
	IOPB_BUFF	vsmd_iopb[NUM_CQES];
	VSMD_CIB	init_info;
	BYTE		bs_slop[CSLOP];
	CRB		vsmd_crb;
	BYTE		bs_slop2[MSLOP];
	BYTE		vsmd_dstatus[4];	/* this cannot move */
	ushort	ipdscr;			/* 1FE,1FF: status change register */
}VSMD_MSHIO;
/* 
 * IOPB TYPES
 */

#define VSCSI_IOPB_TYPE	0		/* for Jaguar 4210 SCSI controller */
#define DISK_IOPB_TYPE	1		/* for 4201/4200 DISK iopbs	   */
					/* MUST use 4-UNIT mode		   */
#define VTAPE_IOPB_TYPE	2		/* for VTAPE emulation on 4201	   */

/*
 * MSR values
 */

#define MSR_CNA		BIT(0)		/* Controller not available	*/
#define MSR_BOK		BIT(1)		/* MACSI Board OK		*/
/*
 * MCR values
 */
#define MCR_SLED	BIT(15)		/* disable status led 		*/
#define MCR_SFEN	BIT(13)

/*
 * CRSW values
 */
#define CRSW_CRBV	BIT(0)		/* Command response block valid	*/
#define CRSW_CC		BIT(1)		/* Command commplete		*/
#define CRSW_ER		BIT(2)		/* Error on last command	*/
#define CRSW_EX		BIT(3)		/* Exception on last command	*/
#define CRSW_FQ		BIT(4)		/* Flush Q complete		*/
#define CRSW_QMS	BIT(5)		/* Q mode started (4210 ONLY)	*/
#define CRSW_CQA	BIT(6)		/* CQE entry available		*/

/*
 * IQAR values
 */
#define IQAR_IQEA	BIT(15)

/*
 * CQE command values
 */
#define CQE_GO		BIT(0)
#define CQE_HPC		BIT(2)		/* run command next	*/
#define CQE_SORT	BIT(7)		/* sort entry into Q	*/

/*
 * MACSI init flags
 */
				/* BIT 0 not used		*/
#define M_OVL_SEEK	BIT(1)	/* enable overlapped seeks	*/
#define M_RESTORE	BIT(2)	/* enable auto-restore on drive faults */
#define M_RROBIN	BIT(3)	/* Force Round-Robin Disk selection */

/*
 * MACSI MACROS
 *
 */

#define VSMD_WSIZ (sizeof(VSMD_IOPB)/sizeof(USHORT))  /* rnds 2 16 bit wrds*/

#define VSMD_CSR(c)     c->ci_device->ipcsr
#define VSMD_DONE_GO(c) \
        VSMD_CSR(c) = (CS_GO | (~(CS_DONE | CS_ERR_LC) & VSMD_CSR(c)))
#define VSMD_DONE(c)    VSMD_CSR(c) &= ~(CS_DONE | CS_ERR_LC)

#define MACSI_CRSW(c)    ((VSMD_MSHIO*)(c->ci_device))->vsmd_crb.crsw
#define MACSI_MSR(c)    ((VSMD_MSHIO*)(c->ci_device))->vsmd_mcsb.msr
#define MACSI_IQAR(c)	 ((VSMD_MSHIO*)(c->ci_device))->vsmd_mcsb.iqar

/* Note: the CQE index was formerly kept in the controller space. There
   is no need for this, since the controller itself never uses it. Further,
   we have experienced problems with occasional bogus 0 values on the
   read of this field in controller space. For security, it is now kept
   in the ctlrinfo structure: however, for the moment, to aid debugging
   the bogus read problem, we will also maintain the value in the controller
   space & check against the value in host memory. */

#define MACSI_CQE_INDX(c) c->ci_qhead

#define MACSI_CQE(c) \
	((VSMD_MSHIO*)(c->ci_device))->vsmd_cqe[MACSI_CQE_INDX(c)]
#define MACSI_QECR(c) \
	MACSI_CQE(c).control
#define MACSI_IOPB_ADDR(c) \
&(((VSMD_MSHIO*)(c->ci_device))->vsmd_iopb[MACSI_CQE_INDX(c)])
#define MACSI_RIOPB_ADDR(c) \
	&(((VSMD_MSHIO*)(c->ci_device))->vsmd_crb.copyiopb)
#define MACSI_CMDTAG(c) \
	 (VSMD_XIOPB*)(((VSMD_MSHIO*)(c->ci_device))->vsmd_crb.cmdtag_l + \
	 (((VSMD_MSHIO*)(c->ci_device))->vsmd_crb.cmdtag_h << 16))

/*----------------------------------------------------------------------*
 *			END of MACSI definitions			*
 *----------------------------------------------------------------------*/
#endif

/*
 * uib (unit initialization block) format
 */
typedef struct ipuib {
	unchar	ipu_v0sh;	/* vol 0 starting head */
	unchar	ipu_v0nh;	/* vol 0 number of heads */
	unchar	ipu_v1sh;	/* vol 1 starting head */
	unchar	ipu_v1nh;	/* vol 1 number of heads */
	unchar	ipu_sectrk;	/* sectors per track */
	unchar	ipu_skew;	/* spiral skew factor */
	unchar	ipu_bytsechi;	/* bytes per sector high */
	unchar	ipu_bytseclo;	/* bytes per sector low */
	unchar	ipu_gap1;	/* words in gap1 */
	unchar	ipu_gap2;	/* words in gap2 */
	unchar	ipu_intrlv;	/* sector interleave factor */
	unchar	ipu_retries;	/* max number of retries on data error */
	unchar	ipu_cylhi;	/* number of cylinders, high */
	unchar	ipu_cyllo;	/* number of cylinders, low */
	unchar	ipu_attrib;	/* attribute flags */
	unchar	ipu_mbz;	/* reserved */
	unchar	ipu_statipl;	/* status change interrupt level */
	unchar	ipu_statvec;	/* status change interrupt vector */
}IPUIB;

/*
 * attribute flags
 * (ipu_attrib)
 */
#define	IPAT_RESEEK	0x01	/* enable recalibrate as last resort */
#define	IPAT_MVBADDATA	0x02	/* enable transfer of bad data */
#define	IPAT_INCBYHEAD	0x04	/* increment by head at track boundry */
#define	IPAT_DUALPORT	0x08	/* observe dual port protocol */
#define	IPAT_STCINTR	0x10	/* enable status change interrupts */
#define	IPAT_CACHEEN	0x20	/* enable sector caching */
#define	IPAT_SPSECEN	0x40	/* enable spare sectoring */
#define IPAT_RUNTSECEN	0x80	/* runt(short) sector enable */

/*
 * reserved attribute flags
 * (ipu_mbz)
 */

#define	RA_MSPARE	BIT(0)	/* multiple spares (uses init long command) */
#define RA_4UNIT	BIT(6)	/* init controller for 4 unit mode	*/
#define ESDI_HARD	0x00	/* ESDI with hard sectors		*/
#define ESDI_SERVO	0x01	/* ESDI with embedded servo		*/
#define ESDI_SOFT	0x10	/* ESDI with soft sectoring		*/


/*
 * misc macros
 */
#define	HI16(x)	((unsigned)(x) >> 16)
#define	LO16(x)	((unsigned)(x) & 0xffff)
#define	HI8(x)	((unsigned)(x) >> 8)
#define	LO8(x)	((unsigned)(x) & 0xff)

#define	IP_MAXIOPBS	32
#define MAX_BADBLOCKS	600
#define MAX_ERRBLOCKS	85
#define DKIPUPC		4			/* 4 drives per controller */


#ifndef NO_MACSI

/*
 * Interface between bufs and IOPB's.  One of these structures will take
 * care of one or more bufs.
 */
struct macsi_command
{
	ushort           flags;
	ushort           nretry;
	struct buf      *firstbuf;
	uint		 firstresid;
	uint             bn;
	uint             length;
	IPSG_FREE       *sg;
	uint             resid;
	struct macsi_command *next;
};

/* status bits for struct macsi_command.flags */
#define MCMD_READ        1
#define MCMD_FULLTRACK   2
#define MCMD_ERROR       4
#endif


/* per unit information */
typedef struct	dkipunitinfo {
	daddr_t	ui_bn;			/* block being transfered */
	unchar	ui_command;		/* command in progress */
					/* (for MACSI mode, # of commands
					   in progress for this unit) */
	uint	ui_stime;		/* For MACSI accounting: time this
					   unit last became active. */
	unchar	ui_attached;		/* non-zero if it attached */
	unchar	ui_vhvalid;		/* non-zero if vh is valid */
	unchar	ui_unit;		/* which unit this is */
	struct	volume_header ui_vh;	/* volume header */
	struct	bst_table *ui_bsttab;	/* bad sector table */
	struct	error_table *ui_errtab;	/* error table */
	ushort	ui_nretries;		/* # of retries for current command */
	uint	ui_spc;			/* sectors per cyl */
	struct	iobuf *ui_tab;		/* pointer to unit table */
	struct	iotime *ui_iotime;	/* pointer to io statistics */

	/* Stuff for special (ioctl) commands */

	struct	buf ui_ctab;		/* command buffer for special cmds */
	IPIOPB	ui_iopb;		/* iopb for special commands	*/
	sema_t	ui_spsem;		/* to serialize use of above */

	IPUIB	ui_uib;			/* uib area for init */
	struct	dkipctlrinfo *ui_ci;	/* back pointer to controller info */
	struct	buf *ui_bp;		/* buffer in progress */
	volatile unsigned char *ui_dstatus;	/* pointer to drive status */
					/* 'volatile' for correct behaviour */
					/* when polling on it. */
	unchar	ui_ilv_drv;		/* interrupt lvl & 4 unit drive num */
}DKIPUNITINFO;

/*
 * Controller BIT flags
 *
 */

#define	CI_SG		BIT(0)		/* do Scatter/Gather		*/
#define CI_WSG		BIT(1)		/* do Word Wide Scatter/Gather	*/
#define	CI_CACHE	BIT(2)		/* Is running Read-Ahead	*/
#define	CI_4UNIT	BIT(3)		/* Is running in 4 Unit mode	*/
#define	CI_MACSI	BIT(4)		/* Is running in MACI mode	*/
#define CI_SORT		BIT(5)		/* Is using Sorting in MACSI	*/
#define CI_OVLSEEK	BIT(6)		/* OVERLAPPED SEEKS in MACSI	*/
#define CI_AUTOREST	BIT(7)		/* AUTO drive clear in MACSI	*/

/* per controller information */
typedef struct	dkipctlrinfo {
	lock_t	ci_lock;		/* MP lock for exclusive use   */
	ulong	ci_sintr;		/* # of spurious interrupts	*/
	ulong	ci_istart;		/* cmds started from interrupt	*/
	ulong	ci_pending;		/* commands pending completion	*/
	ulong	ci_timeouts;		/* # of commands that timed out */
	unchar	ci_sgOK;		/* non-zero if can scat/gath	*/
	unchar	ci_cacheOK;		/* non-zero if can use cache	*/
	unchar	ci_ctlr;		/* controller #			*/
	struct	dkipunitinfo ci_unit[DKIPUPC];	/* unit stuff		*/
	struct	ipiopb ci_iopb[IP_MAXIOPBS];	/* iopbs for big jobs	*/
	volatile struct	ipdevice *ci_device;  /* pointer to i/o registers  */
	struct	iobuf *ci_tab;		/* pointer to controller table	*/
	int	ci_timeid;		/* timeout id			*/

	/* static buffer info. (Note: this buffer is used in non-MACSI
	   mode if a transfer has to cross a page. MACSI can handle this
	   with word-wide scatter/gather). */

	unchar	ci_bufinuse;		/* non-zero if uibuf is in use	*/
	ushort	ci_bufdata;		/* count of bytes in ui_buf	*/
	caddr_t	ci_bufva;		/* kernel vaddr to copy ui_buf	*/
#ifdef R4000
	char	ci_fluffy1[R4K_MAXCACHELINESIZE];
#endif
	long	ci_buf[NBPSCTR/sizeof(long)];	/* static buffer	*/
#ifdef R4000
	char	ci_fluffy2[R4K_MAXCACHELINESIZE];
#endif

	/*
	 * A second buffer for "special" commands (eg flawmap read). There
	 * are MP/semaphoring performance reasons for having a separate 
	 * buffer rather than trying to share the regular one above.
	 */
	long	ci_sbuf[NBPSCTR/sizeof(long)];	/* static buffer	*/
#ifdef R4000
	char	ci_fluffy3[R4K_MAXCACHELINESIZE];
#endif
	sema_t	ci_bufsem;		/* to serialize static buffer use */

	ushort	ci_mode;		/* for NEW style option settings */
	HNDSHK	ci_hndshk;		/* handshake info		*/
#ifndef NOMACSI
	int	ci_qhead;		/* Index of next available CQE	*/
	IPSG_FREE *dkipsg_hd;		/* points to next free entry	*/
	IPSG_FREE dkipsg_fentry[NUM_M_SG]; /* scat/gath free list	*/
#if defined(IP5) || defined(IP17)
	struct cpusg_free *dkipmap_hd;
	struct cpusg_free dkipmap[NUM_M_MAP];
#endif
	struct macsi_command *ci_mcmd_hd; /* pointer to free command struct */
	struct macsi_command ci_mcmd[NUM_MCMD]; /* macsi command structs */
#endif
} DKIPCTLRINFO;

#ifdef	_KERNEL
/*
 * lboot defines the bounds of these data structures
 */
extern	struct iotime dkipiotime[][DKIPUPC];	/* io statistics */
extern	struct dkipctlrinfo dkipctlrinfo[];	/* controller software state */
extern	struct iobuf dkipctab[];		/* controller queues */
extern	struct iobuf dkiputab[][DKIPUPC];	/* drive queues */
extern	int dkipmajor;				/* internal major # */
#endif
