#ifndef __SYS_GMREGS_H__
#define __SYS_GMREGS_H__

/**************************************************************************
 *									  *
 * 		 Copyright (C) 1989, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

#ident "$Revision: 3.8 $"

#if defined(IP5) || defined(IP17)
/*
 * Structure of GM2/GM3/GM3.5 registers on MP BUS
 */
typedef struct gm_ioreg_s {
	uint gm_reg_reset;		/* IOBASE+0: reset the GM (write) */
	uint pad1;			/* IOBASE+4: not used */
	uint gm_reg_status;		/* IOBASE+8: status reg (read only) */
} *gm_ioreg_t;

/*
 * Data type of registers
 */
typedef volatile uint gm_register_t;

#define	GM_MP_BASE_ADDR			0xb7000000
#define GM_MP_TAGR_ADDR			0xb1000000	/* GM2 only */
#define GM_MP_PIPE_ADDR			0xb7300000

#else	/* IP4 */

/*
 * Structure of GM VME registers
 *
 * Writable definitions:
 */
typedef struct gm_ioreg_s {
	unchar pad0;
	unchar gm_reg_reset;		/* IOBASE+1: reset the GM */
	unchar pad2;
	unchar pad3;
	unchar pad4;
	unchar gm_reg_interrupt;	/* IOBASE+5: interrupt the 68020 */
	unchar pad6;
	unchar pad7;
	unchar pad8;
	unchar gm_reg_disable_int;	/* IOBASE+9: disable interrupts */
	unchar pad10;
	unchar pad11;
	unchar pad12;
	unchar gm_reg_int_vec;		/* IOBASE+13: set interrupt vector */
} *gm_ioreg_t;

/*
 * Readable registers:
 */
#define gm_reg_status	gm_reg_reset	/* IOBASE+1: GM status register */

/*
 * Data type of registers
 */
typedef volatile unchar gm_register_t;

#define	GM_VME_BASE_ADDR		0xbd002000	/* in VME A16 space */
#define GM_VME_DRAM_ADDR		0xb8800000	/* in VME A32 space */
#define GM_VME_PIPE_ADDR		0xb8900000	/* in VME A32 space */

#endif	/* IP5 || IP17 */

/*
 * Definition of status register bits
 *	The hardware bits are low true, so the bit masks define the
 *	FALSE conditions:
 */
#define	GM_SR_FIFO_NOT_HALF_FULL	0x01
#define	GM_SR_NOT_RESET			0x02
#define GM_SR_NOT_HALTED		0x04
#define	GM_CONTROLS_FIFO		0x08		/* GM3, GM3.5 */
#define	GM_SR_DATAFIFO_NOT_HALF_FULL	0x10		/* GM3, GM3.5 */
#define	GM_REVISION_MASK		0xe0		/* GM2, GM3, GM3.5 */

#define	GM_REVISION_2			0xa0		/* GM 2 */
#define	GM_REVISION_3			0x00		/* GM 3 */
#define GM_REVISION_35			0x20		/* GM 3.5 */

#endif /* __SYS_GMREGS_H__ */
