#ifndef  __SYS_INVENT_H__
#define  __SYS_INVENT_H__

/*
 * sys/invent.h --  Kernel Hardware Inventory
 *
 * As the system boots, a list of recognized devices is assembled.
 * This list can then be accessed through syssgi() by user-level programs
 * so that they can learn about available peripherals and the system's
 * hardware configuration.
 *
 * The data is organized into a linked list of structures that are composed
 * of an inventory item class and a class-specific type.  Each instance may
 * also specify a 32-bit "state" which might be size, readiness, or
 * anything else that's relevant.
 *
 * In the next major release, the inv_-prefixed, C++ member names of the
 * inventory entry structure will be used by the C language version, and
 * inv_controller and inv_unit will change from char to short type.
 *
 *
 * $Revision: 1.81 $
 *
 * Copyright 1988-1990, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */


typedef struct inventory_s {
#ifdef _LANGUAGE_C_PLUS_PLUS
	struct	inventory_s *inv_next;	/* next inventory record in list */
	int	inv_class;		/* class of object */
	int	inv_type;		/* class sub-type of object */
	char	inv_controller;		/* object major identifier */
	char	inv_unit;		/* object minor identifier */
	long	inv_state;		/* information specific to object or
					   class */
#else
	struct inventory_s *next;	/* next inventory record in list */
	int class;			/* class of object */
	int type;			/* class sub-type of object */
	char controller;		/* object major identifier */
	char unit;			/* object minor identifier */
	long state;			/* information specific to object or
					   class */
#endif
} inventory_t;

typedef struct cpu_inv_s {
	int	cpuflavor;	/* differentiate IP7 and IP15 processor */
	char	cpufq;		/* cpu frequency */
	int	sdsize;		/* secondary data cache size */
} cpu_inv_t;

/* Inventory Classes */
#define INV_PROCESSOR	1
#define INV_DISK	2
#define INV_MEMORY	3
#define INV_SERIAL	4
#define INV_PARALLEL	5
#define INV_TAPE	6
#define INV_GRAPHICS	7
#define INV_NETWORK	8
#define INV_SCSI	9	/* SCSI devices other than disk and tape */
#define INV_AUDIO	10
#define	INV_IOBD	11
#define	INV_VIDEO	12

/* types for class processor */
#define INV_CPUBOARD	1
#define INV_CPUCHIP	2
#define INV_FPUCHIP	3

/* states for cpu and fpu chips are revision numbers */

/* cpuboard states */
#define INV_R2300BOARD	1
#define INV_IP4BOARD	2
#define INV_IP5BOARD	3
#define INV_IP6BOARD	4
#define INV_IP7BOARD	5
#define INV_IP9BOARD	6
#define INV_IP12BOARD	7
#define INV_IP17BOARD	8
#define INV_IP15BOARD   9	/* only prom hinv command use this */
#define INV_IP20BOARD   10
#define INV_IP19BOARD   11
#define INV_IP22BOARD   12

/* types for class INV_IOBD */
#define	INV_CLOVER2	1

/* states for class INV_IOBD type INV_CLOVER2 -- value of board id register */
#define	INV_IO2_REV1	1
#define	INV_IO2_REV2	2
#define	INV_IO3_REV1	3
#define	INV_IO3_REV2	4

/* types for class disk */
/* NB: types MUST be unique within a class.
   Please check this if adding new types. */

#define INV_SCSICONTROL	1
#define INV_SCSIDRIVE	2
#define INV_DKIPCONTROL	3	/* generic: only if not recognised as specific
				   type. Defines for recognised types follow. */
#define INV_DKIPDRIVE	4
#define INV_SCSIFLOPPY	5	/* really any removable scsi disk; includes
				   cdroms, optical disks, etc. */

/* class disk types for specific types of Interphase controllers. */

#define INV_DKIP3200	6
#define INV_DKIP3201	7
#define INV_DKIP4200	8
#define INV_DKIP4201	9
#define INV_DKIP4400	10
#define INV_DKIP4460	14
#define INV_JAGUAR	16
#define INV_VSCSIDRIVE	17

/* class disk types for specific types of Xylogics controllers. */

#define INV_XYL714	11
#define INV_XYL754	12
#define INV_XYL7800	15

/* class disk type for drives on Xylogics controllers */

#define INV_XYLDRIVE	13

/* states for INV_SCSICONTROL disk type; indicate which chip rev;
 * for 93A and B, unit field has microcode rev. */
#define INV_WD93	0	/* WD 33C93  */
#define INV_WD93A	1	/* WD 33C93A */
#define INV_WD93B	2	/* WD 33C93B */

/* class disk state for INV_SCSIFLOPPY type */
#define	INV_TEAC_FLOPPY	1	/* TEAC 3 1/2 inch floppy drive */

/* END OF CLASS DISK TYPES */

/* types for class memory */
/* NB. the states for class memory are sizes in bytes */
#define INV_MAIN	1
#define INV_PROM	2
#define INV_DCACHE	3
#define INV_ICACHE	4
#define INV_WBUFFER	5
#define INV_SDCACHE	6
#define INV_SICACHE	7
#define INV_SIDCACHE	8

/* types for class serial */
#define INV_CDSIO	1	/* Central Data serial board */
#define INV_T3270	2	/* T3270 emulation */
#define INV_GSE		3	/* SpectraGraphics Gerbil coax cable */
#define INV_SI		4	/* SNA SDLC controller */
#define	INV_M333X25 	6	/* X.25 controller */
#define INV_CDSIO_E	7	/* Central Data serial board on E space */
#define INV_ONBOARD	8	/* Serial ports per CPU board */

/* types for class parallel */
#define INV_IKC		1	/* Ikon parallel board  */
#define INV_GPIB	2	/* National Instrument GPIB board */
#define INV_GPIB_E	3	/* National Instrument GPIB board on E space*/

/* types for class tape */
#define INV_SCSIQIC	1	/* Any SCSI tape, not just QIC{24,150}... */
#define INV_XM		2	/* Xylogics 772 1/2 " tape controller */
#define INV_ISIQIC	3	/* QIC-02 interface cartridge tapes */
#define INV_VSCSITAPE	4

/* sub types for type INV_SCSIQIC and INV_VSCSITAPE (in state) */
#define TPUNKNOWN	0	/* type not known */
#define TPQIC24		1	/* QIC24 1/4" cartridge */
#define TPDAT		2	/* 4mm Digital Audio Tape cartridge */
#define TPQIC150	3	/* QIC150 1/4" cartridge */
#define TP9TRACK	4	/* 9 track reel */
#define TP8MM		5	/* 8 mm video tape cartridge */

/*
 *  GFX invent is a subset of gfxinfo
 */

/* types for class graphics */
#define INV_GR1BOARD	1	/* GR1 (Eclipse) graphics */
#define INV_GR1BP	2	/* OBSOLETE - use INV_GR1BIT24 instead */
#define INV_GR1ZBUFFER	3	/* OBSOLETE - use INV_GR1ZBUF24 instead */
#define INV_GRODEV	4	/* Clover1 graphics */
#define INV_GMDEV	5	/* GT graphics */
#define INV_CG2		6	/* CG2 composite video/genlock board */
#define INV_VMUXBOARD	7	/* VMUX video mux board */
#define	INV_VGX		8	/* VGX (PowerVision) graphics */
#define	INV_VGXT	9	/* VGXT (PowerVision) graphics with IMP5s. */
#define	INV_LIGHT	10	/* LIGHT graphics */
#define INV_GR2		11	/* EXPRESS graphics */
#define INV_RE		12	/* RealityEngine graphics */

/* states for graphics class GR1 */
#define INV_GR1REMASK	0x7	/* RE version */
#define INV_GR1REUNK	0x0	/* RE version unknown */
#define INV_GR1RE1	0x1	/* RE1 */
#define INV_GR1RE2	0x2	/* RE2 */
#define INV_GR1BUSMASK	0x38	/* GR1 bus architecture */
#define INV_GR1PB	0x00	/* Eclipse private bus */
#define INV_GR1PBVME	0x08	/* VGR2 board VME and private bus interfaces */
#define INV_GR1TURBO	0x40	/* has turbo option */
#define INV_GR1BIT24  	0x80    /* has bitplane option */
#define INV_GR1ZBUF24 	0x100   /* has z-buffer option */
#define INV_GR1SMALLMON 0x200   /* using 14" monitor */
#define INV_GR1SMALLMAP 0x400   /* has 256 entry color map */
#define INV_GR1AUX4 	0x800   /* has AUX/WID plane option */

/* states for graphics class GR2 */
		/* bitmasks */
#define INV_GR2_Z	0x1	/* has z-buffer option */
#define INV_GR2_24	0x2	/* has bitplane option */
#define INV_GR2_4GE	0x4	/* has 4 GEs */
		/* supported configurations */
#define INV_GR2_XS	0x0     /* GR2-XS */
#define INV_GR2_XSZ	0x1     /* GR2-XS with z-buffer */
#define INV_GR2_XS24	0x2     /* GR2-XS24 */
#define INV_GR2_XS24Z	0x3     /* GR2-XS24 with z-buffer */
#define INV_GR2_XSM	0x4     /* GR2-XSM */
#define INV_GR2_ELAN	0x7	/* GR2-Elan */

/* types for class network */
#define INV_NET_ETHER	0	/* 10Mb Ethernet */
#define INV_NET_HYPER	1	/* HyperNet */
#define	INV_NET_CRAYIOS	2	/* Cray Input/Ouput Subsystem */
#define	INV_NET_FDDI	3	/* FDDI */
#define INV_NET_TOKEN	4	/* 16/4 Token Ring */

/* controllers for network types, unique within class network */
#define INV_ETHER_EC	0	/* IP6 integral controller */
#define INV_ETHER_ENP	1	/* CMC board */
#define INV_ETHER_ET	2	/* IP5 integral controller */
#define INV_HYPER_HY	3	/* HyperNet controller */
#define	INV_CRAYIOS_CFEI3 4	/* Cray Front End Interface, v3 */
#define	INV_FDDI_IMF	5	/* Interphase/Martin 3211 FDDI */
#define INV_ETHER_EGL	6	/* Interphase V/4207 Eagle */
#define INV_ETHER_FXP	7	/* CMC C/130 FXP */
#define INV_FDDI_IPG	8	/* Interphase/SGI 4211 Peregrine FDDI */
#define INV_TOKEN_FV	9	/* Formation fv1600 Token-Ring board */
#define INV_FDDI_XPI	10	/* XPI GIO bus FDDI */
#define INV_TOKEN_GTR	11	/* GTR GIO bus TokenRing */

/* types for class INV_SCSI; should be the same as the device
 * type code returned by the inquiry command.  Note: the
 * lun is encoded in bits 8-15 of the state.  The
 * state field low 3 bits contains the information from the inquiry
 * cmd that  indicates ANSI SCSI 1,2, etc. compliance, and bit 7
 * contains the inquiry info that indicates whether the media is
 * removable. */
#define INV_PRINTER	2	/* SCSI printer */
#define INV_CPU		3	/* SCSI CPU device */
#define INV_WORM	4	/* write-once-read-many (e.g. optical disks) */
#define INV_CDROM	5	/* CD-ROM  */
#define INV_SCANNER	6	/* scanners */
#define INV_OPTICAL	7	/* optical disks (read-write) */
#define INV_CHANGER	8	/* jukebox's for CDROMS, for example */
#define INV_COMM	9	/* Communications device */

/* bit definitions for state field for class INV_SCSI */
#define INV_REMOVE	0x80	/* has removable media */
#define INV_SCSI_MASK	7	/* to which ANSI SCSI standard device conforms*/

/* types for class INV_AUDIO.  Currently there's only one type. */

#define INV_AUDIO_HDSP	0

/* types for class INV_VIDEO */

#define	INV_VIDEO_LIGHT	0


#ifdef _KERNEL

extern void	    add_to_inventory(int, int, char, char, long);
extern void	    *get_next_inventory(void *);
extern inventory_t  *find_inventory(inventory_t *, int, int, char, char, long);
extern int	    scaninvent(int (*)(), void *);

#endif /* _KERNEL */
#endif /* __SYS_INVENT_H__ */
