/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ifndef __SYS_KSA_H__
#define __SYS_KSA_H__
#ident	"$Revision: 3.8 $"
 
/*
 * structure ksa defines the data structure of system activity data file
 */
#include "sys/fs/com_pncc.h"	/* for ncmeter */

struct pnmeter {
	unsigned long	pathnames;	/* number of pathnames created */
	unsigned long	pathbytes;	/* total pathname bytes used */
	unsigned long	longpaths;	/* number of overlong pathnames */
	unsigned long	maxpath;	/* size of longest pathname */
};

/*
 * structs defining private statistics
 */
/*
 * igetstats - stats from iget()/getinode()
 */
struct igetstats {
	unsigned long	ig_attempts;	/* # calls to iget() */
	unsigned long	ig_found;	/* found in hash list */
	unsigned long	ig_fsgone;	/* fs unmounted while getting inode */
	unsigned long	ig_freegets;	/* # times getinode() called */
	unsigned long	ig_oldhashmiss;	/* couldn't get old hash lock */
	unsigned long	ig_ipchg;	/* inode changed while waiting */
};

/*
 * getblk /flush stats
 */
struct getblkstats {
	unsigned long	getblks;	/* # getblks */
	unsigned long	getblockmiss;	/* # times b_lock was missed */
	unsigned long	gethlockmiss;	/* # times hash b_lock couldn't be re-aquired */
	unsigned long	getfound;	/* # times buffer found in cache */
	unsigned long	getbchg;	/* # times buffer chged while waiting */
	unsigned long	getloops;	/* # times back to top of getblk */
	unsigned long	getloops2;	/* # times back to loop2 */
	unsigned long	getloops3;	/* # times back to loop3 */
	unsigned long	getoverlap;	/* # times a buffer overlap was found */
	unsigned long	getfree;	/* # times fell thru to freelist code */
	unsigned long	getfreeempty;	/* # times freelist empty */
	unsigned long	getfreehmiss;	/* # times couldn't get old hash */
	unsigned long	getfreealllck;	/* # times all free bufs were locked */
	unsigned long	getfreedelwri;	/* # times all free bufs were DELWRI */
	unsigned long	flush;		/* # times flushing occurred */
	unsigned long	flushloops;	/* # times flushing looped */
};

/* Statistics on driver locking */
struct drvlock {
	unsigned long		p_indcdev;
	unsigned long		p_indcdevsw;
	unsigned long		p_indbdev;
	unsigned long		p_indbdevsw;
	unsigned long		p_indstr;
	unsigned long		p_indstrsw;
};

struct ksa {
	struct	sysinfo si;	/* defined in /usr/include/sys/sysinfo.h  */
	struct	minfo	mi;	/* defined in /usr/include/sys/sysinfo.h */
	struct	dinfo	di;	/* defined in /usr/include/sys/sysinfo.h */
#ifdef RFS
	struct	rcinfo	ri;	/* defined in /usr/include/sys/sysinfo.h */
#endif
	struct  syserr  se;	/* defined in /usr/include/sys/sysinfo.h */
	struct  pnmeter pmtr;
	struct  ncmeter ncmtr;	/* defined in sys/fs/com_pncc.h */
	struct	igetstats	p_igetstats;
	struct	getblkstats	p_getblkstats;
	struct	drvlock	dlock;
};

#ifdef _KERNEL

#include "sys/pda.h"	/* for private macro */
#include "sys/immu.h"	/* for PDAPAGE, inside private macro */

#define SYSINFO		private.ksaptr->si
#define MINFO		private.ksaptr->mi
#define DINFO		private.ksaptr->di
#define RINFO		private.ksaptr->ri
#define SYSERR		private.ksaptr->se
#define PMTR		private.ksaptr->pmtr
#define NCMETER		private.ksaptr->ncmtr
#define IGETINFO	private.ksaptr->p_igetstats
#define BUFINFO		private.ksaptr->p_getblkstats
#define DRVLOCK		private.ksaptr->dlock

#endif /* _KERNEL */
#endif /* __SYS_KSA_H__ */
