#ifndef __SYS_LG1_H__
#define __SYS_LG1_H__

/*
 *  $Revision: 1.38 $
 */

#if	defined(IP17)
#define GFX_NAME_LIGHT		"LG1MP"	/* Name for Crimson LIGHT board */
#endif
#if	defined(IP20) || defined(IP22)
#define GFX_NAME_LIGHT		"LG1MC"	/* Name for Indigo R4K LIGHT board */
#define LG1MC
#endif
#if	defined(IP12)
#define GFX_NAME_LIGHT		"LG1"	/* Name for Indigo R3K LIGHT board */
#endif
#if	!defined(GFX_NAME_LIGHT)
#define GFX_NAME_LIGHT		"LG1"	/* Name for LIGHTNOHW */
#endif

struct lg1_info {
			/* device independent information */
	struct gfx_info gfx_info; 
			/* lg1 specific information */
	unsigned char boardrev;
	unsigned char rexrev;
	unsigned char vc1rev;
		/* next 2 only supported for boardrev >= 1 */
	unsigned char monitortype;
	unsigned int videoinstalled;
		/* only valid for IP12 */
	unsigned char picrev;
};

#define LIGHT_SYNC_VALUE 4

#ifdef _KERNEL

#define LIGHT_3WAY_SIZE 0x1000
#define LIGHT_REX_SIZE  0x5000
#define LIGHT_MAP_SIZE 	(LIGHT_3WAY_SIZE + LIGHT_REX_SIZE) /* 6 pages */

struct lightData {
	struct gfx_data	gfx_data;	/* Common gfx driver above */
	struct rexchip	*rex;		/* K1 seg board base address */
	struct lg1_info *info;
	unsigned int 	attaches;
	void *		Retrace;
	void *		dmadap;		/* space for DMA descriptor array */
	unsigned short 	cursor_xhot;
	unsigned short 	cursor_yhot;
	unsigned long 	loadedmodes[32];
	unsigned int 	cursor_x;
	unsigned int 	cursor_y;
	int		dmainprogress;
	sema_t		dma;
#ifdef IP17
	unsigned void   *adapter_base;	/* kernel address of base address of adapter board */
	int		adapter_slot;	/* MP slot id of adapter (only on MP bus systems)*/
	int		gio_slot;	/* MP slot id of GIO (only on MP bus systems)*/
#endif
};

/*
 * device dependent rendering node
 */
struct light_rnode {
	short xorg, yorg;	/* geometry at last clip */
	short xsize, ysize;
	short smask_llx, smask_lly; /* GL screen mask */
	short smask_urx, smask_ury;
#ifdef IP17
	unsigned long 		ip17p3way_cntcmdreg; /* only 16 bits are valid */
	unsigned long		ip17p3way_ABCDregs[4];
#endif
};

struct light_cx {
	unsigned long current_xstart;
	unsigned long current_ystart;
	unsigned long current_colorred;
	unsigned long current_colorgreen;
	unsigned long current_colorblue;
	unsigned long current_lspattern;
	unsigned long current_lsmode;
	unsigned long current_zpattern;
	unsigned long current_xsave;
	unsigned long current_rwaux1;
	unsigned long next_xstart;
	unsigned long next_ystart;
	unsigned long next_colorred;
	unsigned long next_colorgreen;
	unsigned long next_colorblue;
	unsigned long next_slopered;
	unsigned long next_slopegreen;
	unsigned long next_slopeblue;
	unsigned long next_xend;
	unsigned long next_yend;
	unsigned long next_aux1;
	unsigned long next_lspattern;
	unsigned long next_lsmode;
	unsigned long next_zpattern;
	unsigned long next_xsave;
	unsigned long next_minorslope;
	unsigned long next_xymove;
	unsigned long next_colorback;
	unsigned long next_aweight;
	unsigned long next_rwaux1;
	unsigned long next_rwmask;
	unsigned long next_command;
	unsigned long smask1x;
	unsigned long smask1y;
		/* configregs */
	unsigned long xywin;
	unsigned long aux2;
	unsigned long configmode;
#ifdef IP17
	unsigned long mg1_control1;
	unsigned long mg1_control2;
#endif
};


/*
 *  REXWAIT macros
*/
	/* wait for rex fifo to drain and chip to idle */


	/* wait for all fifos to drain and chip to idle */
	/* need to test twice to avoid 5 clock pulse in GIO_FF */
#ifdef IP17
#define REXWAIT(rex)										\
	while ( ((*(volatile long *)(PHYS_TO_K1(MG1_STATUS)) & \
		(MG1_STAT_ADP_FE | MG1_STAT_GFXDLY)) != (MG1_STAT_ADP_FE))); \
	while ( ((*(volatile long *)(PHYS_TO_K1(MG1_STATUS)) & \
		(MG1_STAT_ADP_FE | MG1_STAT_GFXDLY)) != (MG1_STAT_ADP_FE))); \
	while ((rex)->p1.set.configmode & CHIPBUSY);
#define BFIFOWAIT REXWAIT
#else
#define REXWAIT(rex) 	while ((rex)->p1.set.configmode & CHIPBUSY);
#define BFIFOWAIT(rex) 	while ((rex)->p1.set.configmode & BFIFO);
#endif

#endif /* _KERNEL */

/*
 *   light private ioctls
 */
#define LG1_BASE			20000
#define LG1_SET_CURSOR_HOTSPOT		(LG1_BASE+1)
#define LG1_PIXELDMA			(LG1_BASE+2)
#define LG1_SET_DESTINATION_PLANES	(LG1_BASE+3)
#define LG1_SET_SCREENMASK		(LG1_BASE+4)
#define LG1_GET_GEOMETRY		(LG1_BASE+5)
#define LG1_READDISPLAY                 (LG1_BASE+6)

struct lg1_set_cursor_hotspot {
	unsigned short xhot;
	unsigned short yhot;
};

struct lg1_pixeldma_args {
	long	xlen;			/* number of pixels (in bytes) to
					   be DMA'ed */
	long	ylen;			/* number of lines */
	long	flags;
					/* LG1_WRITE, LG1_STRIDE, LG1_ZOOM */
        long pmstride;                  /* Number of bytes in stride */ 
	void *gfxaddr;			/* Graphics address to DMA to/from */
        char *buf;                      /* Data buffer */
};

/* Flags for graphics DMA */
#define LG1_WRITE       0x01            /* DMA direction: host to gfx */
#define LG1_STRIDE      0x02            /* Stride DMA */
#define LG1_ZOOM        0x04            /* Pixel Zooming */

#define LG1_DMA_MAXSIZE (256*1024) 	/* 256K limit */
#define LG1_DMA_MAXXLEN	(1024*4)
#define LG1_DMA_MAXYLEN	(768)

struct lg1_set_destination_planes {
	unsigned long planes; /* bits to set in aux2 */
};

struct lg1_set_screenmask {
	short llx, lly;
	short urx, ury;
};

struct lg1_get_geometry {
	long xorigin, yorigin;
	long xsize, ysize;
};

struct lg1_readdisplay {
	char colorMaps[0xc00];
	char auxMapIndex;
	unsigned short displayModes[0x40];
	unsigned short scans[0x8000];
};

/* 
 *  memory layout of vc1 sram
 */
#define VC1_VID_LINE_TAB_ADDR		0x0
#define VC1_VID_FRAME_TAB_ADDR		0x800
#define VC1_CURSOR_GLYPH_ADDR           0x700

#endif /* __SYS_LG1_H__ */

