#ident	"$Revision: 3.32 $"

/*
 * Copyright 1985 by MIPS Computer Systems, Inc.
 */

#ifndef __SYS_MTIO_H__
#define __SYS_MTIO_H__

/*	mtio.h	6.1	83/07/29	*/

/*
 * Structures and definitions for mag tape io control commands
 */

/* structure for MTIOCTOP - mag tape op command */
struct	mtop	{
	short	mt_op;		/* operations defined below */
	ulong	mt_count;	/* how many of them */
};

/* operations (mt_op values for MTIOCTOP) */
#define MTWEOF	0	/* write an end-of-file record */
#define MTFSF	1	/* forward space file */
#define MTBSF	2	/* backward space file */
#define MTFSR	3	/* forward space record */
#define MTBSR	4	/* backward space record */
#define MTREW	5	/* rewind */
#define MTOFFL	6	/* rewind and put the drive offline */
#define MTNOP	7	/* no operation, sets status only */
#define MTRET	8	/* retention operation */
#define MTRST	9	/* reset operation */
#define MTEOM	10	/* space to end of recorded data */
#define MTAFILE	11	/* space to end of last file before FM (so next data
	written is appended) */
#define MTERASE	12	/* erase tape from current position to EOT */
#define MTUNLOAD 13	/* unload tape from drive */

#define MTSETPART	14	/* skip to partition in mt_count (DAT); for DDS
	* format DAT drives (only ones supported current), partition 1 is the first
	* partition on the tape (nearest BOT), and partition 0 is the remainder of
	* the tape. */
#define MTMKPART	16	/* create partition (DAT), count is multiplied by
	* 1 Mybte (2^20 bytes) to give the size of partition 1; if 0, then changes
	* tape to a single partition tape. */
#define MTSKSM	17	/* skip setmark (DAT); skip backwards if mt_count is
	negative */
#define MTWSM	19	/* write mt_count setmarks (DAT) */
#define MTAUD	20	/* turn audio mode on and off.  If count is 0, we
	* are in data mode (default), if 1, put drive in audio mode.  This is
	* only needed for writing tapes, since when reading the tape type
	* (data or audio) is recognized automaticly. (DAT only for now).
	* Remains set until same ioctl is reused, or until reboot. */
#define MTSEEK	21	/* Seek to the given block number.  Not all drives
	* support this option.  For drives that support audio, the block
	* number actually becomes the program number ( 0 meaning leadin
	* area, 0xfffffffe meaning leadout, and other values (currently
	* 1-799 decimal) the program #).  For seeking to times in audio mode,
	* see MTSETAUDIO. */
	/*OLSON: use bbb and eee as for get? */


/* structure for MTIOCGET - mag tape get status command; note that
 * in audio mode, the blkno is actually the program number; see
 * also the MTGETAUDIO ioctl and structure.  For the 3 special
 * position in audio mode (leadin, leadout, and unknown), the
 * low nibbles are replicated, so they don't match valid program
 * numbers (which are 1-799 decimal), giving values of 0xBBB, 0xEEE,
 * and 0xAAA, respectively.
*/
struct	mtget	{
	short	mt_type;	/* type of magtape device */
	unsigned short	mt_dposn;	/* status of tape position (see below) */
/* the following two registers are device dependent */
	unsigned short	mt_dsreg;	/* ``drive status'' register */
	short	mt_erreg;	/* ``error'' register */

	short	mt_resid;	/* residual count; for SCSI, this is used
		to report the current partition # for DAT tapes that are
		partitioned. */
	/* the following two are not yet fully implemented */
	daddr_t	mt_fileno;	/* file number of current position */
	daddr_t	mt_blkno;	/* block number of current position */
};

/* old mtget structure , still keep it around for compatibility reason */
/* the librmt and /etc/rmt code uses it */
struct	old_mtget	{
	short	mt_type;	/* type of magtape device */
/* the following two registers are device dependent */
	unsigned short	mt_dsreg;	/* ``drive status'' register */
	short	mt_erreg;	/* ``error'' register */

	short	mt_resid;	/* residual count */
	/* the following two are not yet fully implemented */
	daddr_t	mt_fileno;	/* file number of current position */
	daddr_t	mt_blkno;	/* block number of current position */
};


/*	structure for MTIOCGETBLKINFO - all quantities in bytes.
	A value of 0 indicates no limit
	When using the variable block size device, the lastread
	field is determined by reading one block and then spacing
	backwards, if no i/o has yet been done to this tape.  Unlike 
	MTIOCGETBLKSIZE, all quantities are in bytes.
*/
struct mtblkinfo {
	unsigned minblksz;	/* minimum block size */
	unsigned maxblksz;	/* maximum block size */
	unsigned curblksz;	/* size of block at last i/o.  For fixed
		mode, all requests must be a multiple of this. */
	unsigned recblksz;	/* "recommended" size for i/o.  Mostly
		based on compatibility with earlier releases and/or
		other vendors */
};

/* structure and defines for MT[GS]ETAUDIO.
 * When using SETAUDIO to locate to a particular point, the type
 * should be set to one of the MTAUDPOSN_* values, 
 * and only the corresponding fields need be filled in.  See the DAT
 * audio spec for more detail on the definitions of the different times.
 * There are 34 frames per second, legal values are 0 through 33.  The
 * maximum value for the hours field is 99 hours.  Positioning commands
 * return immediately (as does rewind) in audio mode, and requests for
 * current position may be made while the positioning is in progress.
 * Other commands will block while positioning is in progress.
 * When known, all 4 of the positions are returned for GET (that is
 * pnum1-index, and all 3 of the mtaudtime structures).  The program
 * must determine by looking for 0xA in both nibbles whether any particular
 * field is valid.
*/

#define MTAUDPOSN_PROG	0	/* seek to program number */
#define MTAUDPOSN_ABS 	1	/* seek to absolute time */
#define MTAUDPOSN_RUN 	2	/* seek to running time */
#define MTAUDPOSN_PTIME 3	/* seek to program time (within program) */


/*
 * all the times are BCD digits, for both SET and GET.
 * a value of 0xA in BOTH nibbles, (i.e., not a valid BCD number)
 * means that the field isn't valid.
 */
struct mtaudtimecode {
    unchar hhi:4, hlo:4;	/* hours */
    unchar mhi:4, mlo:4;	/* minutes */
    unchar shi:4, slo:4;	/* seconds */
    unchar fhi:4, flo:4;	/* frame # (finer grain than seconds) */
};

struct mtaudio {
    unchar zero1:4, pno1:4; /* 3 BCD digits giving program number the zero */
    unchar zero2:4, pno2:4; /* fields should always be 0, or 0xA, when the */
    unchar zero3:4, pno3:4; /* pno field is also 0xA, indicating not valid */
    unchar indexhi:4, index:4; /* index number within program; hi is the
		most significant BCD digit, index the least. */
    struct mtaudtimecode ptime;         /* program time */
    struct mtaudtimecode atime;         /* absolute time */
    struct mtaudtimecode rtime;         /* running time */
    unchar seektype;            /* one of the MTAUDPOSN_* values */
    unchar cfuture[11];  /* future expansion */
};


/* structure for MTCAPABILITY */
struct mt_capablity  {
	int mtsubtype;	/* this is NOT the same as mt_type, this is
		the actual tape type, from invent.h, such as 9track, or DAT;
		otherwise it is difficult to associate a particular device
		with its type from a program. */
	uint mtcapablity;	/* the MTCAN* bits */
	uint mtfuture[8];	/* for future additions */
};

/* Constants for mt_type byte */
#ifdef mips
#define MT_ISQIC	0x01		/* qic-2 tape controller */
#define MT_ISXM		0x08		/* Xylogics magtape controller */
#define MT_ISSCSI	0x09		/* Western Digital SCSI controller */
#define MT_ISVSCSI	0x0A		/* Interphase VME-SCSI controller */
#else
#define MT_ISTS		0x01
#define MT_ISHT		0x02
#define MT_ISTM		0x03
#define MT_ISMT		0x04
#define MT_ISUT		0x05
#define MT_ISCPC	0x06
#define MT_ISAR		0x07
#endif


/*	values for mt_dposn; values chosen for compatiblity with
	existing programs!  Note that these bits and the gaps between
	are also used in tpsc.h */
#define MT_EOT		0x01	/* tape is at end of media */
#define MT_BOT		0x02	/* tape is at beginning of media */
#define MT_WPROT	0x04	/* tape is write-protected */
#define MT_ONL		0x40	/* drive is online */
#define MT_EOD		0x4000	/* tape is at end of data */
#define MT_FMK		0x8000	/* tape is at file mark */
#define MT_POSMSK	0xC047	/* mask for supported bits */

/* bits in mt_dsreg; set only for drives that support QIC150 as well
	as the older formats.  Specific to SCSI tape */
#define MT_QIC24 0x80	/* QIC24 cartridge in drive */
#define MT_QIC120 0x100	/* QIC120 cartridge in drive */

/* mag tape io control commands */
#define MTIOCODE(x)	('t'<<8|(x))
#define MTIOCTOP	MTIOCODE('a')		/* perform tape op */
#define MTIOCGET	MTIOCODE('b')		/* get tape status */
#define MTIOCGETBLKSIZE	MTIOCODE('c')		/* get tape block size
	in multiples of 512 bytes */
#define MTSCSIINQ	MTIOCODE('d')
#define MTSPECOP	MTIOCODE('e')		/* special drive specific ops */
#define MTIOCGETBLKINFO	MTIOCODE('f')		/* get tape block info */
#define MTANSI    MTIOCODE('g')           /* allows i/o past EOT mark to
	provide for ANSI 3.27 labeling.  Doesn't persist across
	opens.  If used, standard SGI EOT handling for tar, bru, etc.
	won't work correctly.  Currently implemented only on SCSI
	tape drives.  An arg of 1 enables, 0 disables.  NOTE: when
	the EOT marker is encountered, the current i/o operation
	returns either a short count (if any i/o completed), or -1
	(if no i/o completed); it is the programmers responsiblity to
	determine if EOT has been encountered by using MTIOCGET or
	logic internal to the program.  This ioctl should be issued
	AFTER encountering EOT, if ANSI 3.27 EOT handling is
	desired.  subsequent i/o's will then be allowed, and will
	return the count actually transferred, or -1 if the drive was
	unable to transfer any data.  The standard calls for writing
	a FM before the label.  If this isn't done, the drive may
	return label info as data.  */
#define MTCAPABILITY    MTIOCODE('h')  /* returns drive tape
	and capablities (MTCAN*).  Currently implemented only
	for SCSI drives.  See mt_capablity struct */
#define MTSETAUDIO    MTIOCODE('i')  /* sets current
		position, etc. in audio mode */
#define MTGETAUDIO    MTIOCODE('j')  /* returns info about
		position, etc. in audio mode */
#define MTABORT    MTIOCODE('k')  /* abort current tape operation,
	if any.  Useful only with commands that run asynchronously
	to the caller, such as seek and rewind when in audio mode. */



/* MTSPECOP - special drive specific operations.  These use the mtop
	structure. */
#define MTSCSI_CIPHER_SEC_ON	1 /* enable reporting of recovered
				   errors on the Cipher 540S */
#define MTSCSI_CIPHER_SEC_OFF	2	/* disable recovered error report */

/* These are meaningful only for Exabyte.  Default is to return
	errors if read length is less than the block length */
#define MTSCSI_SILI		3	/* suppress illegal length indication */
#define MTSCSI_EILI		4	/* enable illegal length intication */

/* currently implemented only for SCSI tape driver */
#define MTSCSI_SETFIXED		5	/* set size of blocks for fixed block
	device in mt_count.  Lasts till reset same way, or till next boot. */
#define MTSCSI_SPEED		6	/* set speed for multispeed devices in
	mt_count.  Current legal values are 0 (50 ips) and 1 (100 ips)
	on Kennedy drives only.  Default is currently 100. */

#define MTSCSI_SM	7	/* How DAT setmarks are handled.  default
	is to treat similar to FM (also if non-zero in mt_count).  0
	in mt_count causes setmarks to be ignored.  Lasts until
	reboot or reset. */

#define MTSCSI_INQUIRY_LEN 82	/* length of inquiry data for scsi tapes */

/* definitions for tape capablities.  Only those that are not
 * shared by all tape drives are shown. */
#define MTCAN_BSF	0x1	/* can backspace file */
#define MTCAN_BSR	0x2	/* can backspace record (block) */
#define MTCAN_APPEND	0x4	/* can append to existing tape files */
#define MTCAN_SETMK	0x8	/* can do setmarks */
#define MTCAN_PART	0x10	/* can do multiple partitions (DAT) */
#define MTCAN_PREV	0x20	/* can prevent/allow media removal */
#define MTCAN_SYNC	0x40	/* can do synchronous mode SCSI */

/* Note.  If SPEOD are set, or just SPEOD is set,
 * then a simple space to EOD is done; the combination
 * of SPEOD and LEOD is meaningless, and is treated as SPEOD, when
 * spacing, or as LEOD when closing; i.e., do NOT set both, or you
 * will get inconsistent results.
 * Some drives, such as the 8mm drive, have a drive detectible EOD,
 * but don't support the space EOD command.  For these drives,
 * neither SPEOD nor LEOD should be set.  We then fake it by looping
 * on space record and space FM until we get a blank check or other
 * indication of EOD. */
#define MTCAN_LEOD	0x80	/* Use a logical EOD (as opposed to an EOD
	* detectable by the drive).  Currently only 9-track drives.
	* If set, it is assumed EOD is marked by 2 sequential filemarks.
	* Setting this also implies causes two sequential FM's to be written
	* on close, when writing. */
#define MTCAN_SPEOD	0x100	/* can execute the space EOD command. */

#define MTCAN_CHKRDY 0x200	/* can determine if a tape is present; some
	* drives, such as Cipher540S return OK on a test unit ready even if
	* no cartridge is present. */

#define MTCAN_VAR	0x400	/* can do variable block sizes */
#define MTCAN_SETSZ	0x800	/* can set fixed block size */
#define MTCAN_SETSP	0x1000	/* can set tape speed (some 9track) */
#define MTCAN_SETDEN	0x2000	/* can set tape density (some 9track) */
#define MTCAN_SILI	0x4000	/* can set suppress illegal length errors;
	* only on drives that support variable block sizes */
#define MTCANT_RET	0x8000	/* some drives can't retension; the driver
	* will do it's best by spacing to EOD and rewinding. */
#define MTCAN_AUDIO	0x10000	/* drive supports audio over the SCSI bus;
	* only DAT at this time. The data is expected to be read
	* or written in the standard DAT format, including subcodes.
	* collection and gathering of subcodes is handled by the drive.
	* all 7 subcodes follow the data, then the mainID.
	* The drive reports/sets audio or data tapes via the density code
	* in the modesense/select block descriptor.  Actual data layout
	* is as below:
	*   5760 bytes of audio data (16 bits left, 16 bits right, repeat)
	*   7 subcode packs of 8 bytes each
	*   4 bytes of subid (4bits each of ctrlid, dataid, program # high
	*     nibble, # of valid pack items, progno # middle nibble, progno
	*     low nibble, 2 reserved nibbles)
	*   2 bytes main id (2 bits each of fmtid, and id1 - id7)
	*/
#define MTCAN_SEEK	0x20000	/* drive supports seeking to a particular
	* block (or sometimes program # if AUDIO supported) */
#define MTCAN_CHTYPEANY 0x40000	/* drive can change density
	and/or fixed to variable at any point (if it can change at all).
	Otherwise it can change only at BOT */
#endif /* __SYS_MTIO_H__ */
