#ifndef __SYS_PAR_H__
#define __SYS_PAR_H__
#ifdef __cplusplus
extern "C" {
#endif
/**************************************************************************
 *									  *
 * 		 Copyright (C) 1989, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/
#ident "$Revision: 1.16 $"

/* par operation types values given to /dev/par(ioctl) */

#define PAR_START	100	/* par start */
#define PAR_READ	102	/* read par buffer */
#define PAR_SYSCALL	103	/* trace system calls  */
#define PAR_RESCHED	104	/* trace scheduler  */
#define PAR_NETSCHED	105	/* trace network scheduling */
#define PAR_NETFLOW	106	/* trace network data flow */

/* storage format tokens for fawlty */
#define OLDPROCTKN	3	/* resched -- old process token */
#define NEWPROCTKN	4	/* resched -- new process token */
#define IDLEPROCTKN	5	/* resched -- idle process token */
#define USTRINGTKN	6	/* user string token */
#define SYSCALLTKN	8	/* system call token */
#define SYSERRTKN	9	/* system call error token */
#define RUNQTKN		11	/* resched -- runq  token */
#define SYSRETTKN	12	/* system call return code token */
#define OFLOWTKN	13	/* overflow count */
#define NAMETKN		14	/* process name token */
#define RMPROCTKN	15	/* remove died process token */
#define NETEVENTKN	16	/* net new event inited */
#define NETSLEEPTKN	17	/* net sleeps */
#define NETWAKEUPTKN	18	/* net waken up */
#define NETWAKINGTKN	19	/* net waking up */
#define NETFLOWTKN	20	/* net data flow */
#define NETDROPTKN	21	/* net data drop */
#define NETEVENTDONETKN	22	/* net event done */
#define SENDSIGTKN	23	/* process is sending (delivering) signal */
#define RCVSIGTKN	24	/* process is receiving (processing) signal */

#define STRINGTKN1	0x1	/* one string parameter */
#define STRINGTKN2	0x2	/* two string parameters */

/* reasons for idle */
#define NO_STACK	0x1	/* kernel stack is used on another processor */
#define NO_SLOAD	0x2	/* not in core */
#define MUSTRUN		0x4	/* only allowed to run on certain processor */
#define NO_FP		0x8	/* FP on another processor */
#define NO_GFX		0x10	/* graphics lock held by another process */
#define EMPTY		0x20	/* run queue empty */
#define NO_DISC		0x40	/* cpu doesn't run right sched disciplines */
#define IN_SINGLE	0x80	/* wrong proc when in single proc mode */
#define NO_GANG		0x100	/* proc member of gang that can't schedule */
#define GANG_PREEMPTED	0x200	/* gang is queued but better ndpri guy */

/* reasons for resheduled */
#define MUSTRUNCPU	0x01	/* must run on other cpu */
#define SEMAWAIT	0x02	/* wait for semaphore */
#define RESCHED_P	0x04	/* rescheduled due to preempt */
#define RESCHED_Y	0x08	/* rescheduled due to yield */
#define RESCHED_S	0x10	/* rescheduled due to stopped by signal */
#define RESCHED_D	0x20	/* rescheduled due to process died */

/* reasons for netevents */
#define NETRES_NULL	0x00	/* null reason */
#define NETRES_MBUF	0x01	/* mbuf */
#define NETRES_QFULL	0x02	/* queue full */
#define NETRES_QEMPTY	0x03	/* queue empty */
#define NETRES_SBFULL	0x04	/* socket buffer full */
#define NETRES_SBEMPTY	0x05	/* socket buffer empty */
#define NETRES_CKSUM	0x06	/* checksum */
#define NETRES_SIZE	0x07	/* bad size packet */
#define NETRES_HEADER	0x08	/* bad packet header */
#define NETRES_FRAG	0x09	/* duplicated fagment */
#define NETRES_SYSCALL	0x0a	/* system call */
#define NETRES_INT	0x0b	/* interrupt */
#define NETRES_ERROR	0x0c	/* error */
#define NETRES_PROTCALL	0x0d	/* protocol call */
#define NETRES_PROTRET	0x0e	/* protocol return */
#define NETRES_UNREACH	0x0f	/* unreachable */
#define NETRES_CONN	0x10	/* connect call */
#define NETRES_CONNDONE	0x11	/* connect done */
#define NETRES_OFFSET	0x12	/* bad offset */
#define NETRES_NOTOURS	0x13	/* forwarding datagram */

#define	NETRES_RFSNULL	0x14
#define	NETRES_GETATTR	0x15
#define	NETRES_SETATTR	0x16
#define	NETRES_ROOT	0x17
#define	NETRES_LOOKUP	0x18
#define	NETRES_READLINK	0x19
#define	NETRES_READ	0x1a
#define	NETRES_WRTCACHE	0x1b
#define	NETRES_WRITE	0x1c
#define	NETRES_CREATE	0x1d
#define	NETRES_REMOVE	0x1e
#define	NETRES_RENAME	0x1f
#define	NETRES_LINK	0x20
#define	NETRES_SYMLINK	0x21
#define	NETRES_MKDIR	0x22
#define	NETRES_RMDIR	0x23
#define	NETRES_READDIR	0x24
#define	NETRES_STATFS	0x25
#define	NETRES_NPROC	0x26

#define NETRES_SBLOCK	0x27	/* sb lock */
#define NETRES_SBUNLOCK	0x28	/* sb unlock */
#define NETRES_SBFLUSH	0x29	/* sb flush */
#define NETRES_TIMO	0x29	/* sb flush */
#define NETRES_BUSY	0x30	/* busy */

/* netevent subtoken */
#define NETEVENT_LINKUP		0x00
#define NETEVENT_LINKDOWN	0x01
#define NETEVENT_IPUP		0x02
#define NETEVENT_IPDOWN		0x03
#define NETEVENT_UDPUP		0x04
#define NETEVENT_UDPDOWN	0x05
#define NETEVENT_TCPUP		0x06
#define NETEVENT_TCPDOWN	0x07
#define NETEVENT_SOUP		0x08
#define NETEVENT_SODOWN		0x09
#define NETEVENT_NFSUP		0x0a
#define NETEVENT_NFSDOWN	0x0b
#define NETEVENT_RAWUP		0x0c
#define NETEVENT_RAWDOWN	0x0d
#define NETEVENT_ARPUP		0x0e
#define NETEVENT_ARPDOWN	0x0f
#define NETEVENT_RPCUP		0x10
#define NETEVENT_RPCDOWN	0x11
#define NETEVENT_DDNUP		0x12
#define NETEVENT_DDNDOWN	0x13

/* misc for netevent */
#define NETCNT_NULL	((char)-99)
#define NETPID_NULL	((char)-98)

#ifdef _KERNEL
extern int trflags;
extern void fawlty_exit(void);
extern void fawlty_fork(void);
extern void fawlty_rcvsig(int);
extern void fawlty_rename(void);
extern void fawlty_sendsig(int, pid_t);
extern void fawltyidle(char, int);
extern void fawltynetevent(char, pid_t, char, int, char);
extern void fawltysched(char, struct proc *, char, char, int);
extern void fawltysys(short, struct sysent *, int *);
extern void fawltysysend(short, int, int);

#ifdef NET_DEBUG
#define NETPAR(f,t,p,s,l,r) {if (trflags&(f)) fawltynetevent(t, p, s, l, r);}
#else
#define NETPAR(f,t,p,s,l,r)
#endif /* NET_DEBUG */

/* fault trace flags - kept in trflags */
#define SYSCALLHIST	0x04	/* trace system calls */
#define RESCHEDHIST	0x08	/* trace rescheds */
#define ALLPROCSHIST	0x20	/* track all processes */
#define NETSCHED	0x40	/* trace network scheduling */
#define NETFLOW		0x80	/* trace network data flow */

#endif /* _KERNEL */

/*
 * Fawlty record definitions kernel -> padc
 */

#define FAWLTMAX	(4*1024 - 2)	/* sizeof(struct fawltybuf) == 16k */
/*
 * Fawlty trace buffer structure
 */
struct fawltybuf {
	int	stat;		/* current status of this buffer */
	int	len;		/* length of current contents in ints */
	int	buf[FAWLTMAX];	/* buffer contents */
};

#define TOKENSIZ	8	/* each fawlty rec contains 8 bit token field */
#define BYTESIZ(strct)	(roundup(sizeof(strct),sizeof(int)))

/*
 * Buffer structure to describe a string parameter.
 */
struct stringbuf {
	int	len;	/* length in bytes */
	caddr_t	ptr;	/* pointer to string */
};
typedef struct stringbuf strbuf_t;

/*
 * Net event record: used by NETEVENTKN, NETSLEEPTKN, NETWAKEUPTKN, 
 * NETWAKINGTKN, NETFLOWTKN, NETDROPTKN, and NETEVENTDONETKN tokens.
 */
struct neteventrec {
	short	pid;
	unchar	subtkn;
	unchar	token;
	uint	counts;
	ushort 	pad;
	unchar	reasons;
	unchar 	cpu;
	uint	tick;
};
typedef struct neteventrec neteventrec_t;

/*
 * Schedule record: used by OLDPROCTKN, NEWPROCTKN, and RUNQTKN tokens.
 */
struct schedrec {
	short	pid;
	unchar	pri;
	unchar	token;
	uint	reasons;
	ushort	pad;
	unchar	rtpri;
	unchar	cpu;
	uint	event;
	uint	event2;
	uint	tick;
};
typedef struct schedrec schedrec_t;

/*
 * Idle record: used by IDLEPROCTKN token.
 */
struct idlerec {
	short	cpu;
	unchar	reasons;
	unchar	token;
	uint	tick;
};
typedef struct idlerec idlerec_t;

/*
 * System call record: used by SYSCALLTKN token.
 */
#define FAWLTY_MAX_PARAMS	6
/*
 * NOTE: kernel and padc code depends upon the fact that the size of the
 * callrec struct preceeding the params array may be copied by integer
 * (that is, 4,8,12, etc. bytes long).  All hell will break loose if a
 * field is added before params that makes it byte-aligned (if the compiler
 * permits it without rounding).
 */
struct callrec {
	short	pid;
	unchar	numparams;
	unchar	token;
	ushort	callno;
	unchar	cpu;
	unchar	strings;
	uint	tick;
	uint	params[FAWLTY_MAX_PARAMS];
};
typedef struct callrec callrec_t;

/*
 * System call record: used by SYSERRTKN token.
 */
struct callendrec {
	short	pid;
	unchar	cpu;
	unchar	token;
	ushort	callno;
	ushort	pad;
	uint	retval;
	uint	error;
	uint	tick;
};
typedef struct callendrec callendrec_t;

/*
 * Process-name record: used by NAMETKN.
 */
struct namerec {
	short pid;
	unchar pad;
	unchar token;
	unchar name[16];
};
typedef struct namerec namerec_t;

/*
 * Remove-process record: used by RMPROCTKN.
 */
struct rmprocrec {
	short	pid;
	ushort	token;
};
typedef struct rmprocrec rmprocrec_t;

/*
 * Signal record: used by SENDSIGTKN and RCVSIGTKN.
 */
struct sigrec {
	short	rcvpid;
	unchar	cpu;
	unchar	token;
	ushort	signo;
	uint	tick;
};
typedef struct sigrec sigrec_t;

/*
 * User supplied string record
 */
typedef struct ustringrec {
	short	pid;
	unchar	pad;
	unchar	token;
	char	string[64];
	uint	tick;
} ustringrec_t;
#ifdef __cplusplus
}
#endif
#endif /* __SYS_PAR_H__ */
