/*
 * $Revision: 1.8 $
 */

#ifndef __SV1_STRUCT_H_
#define __SV1_STRUCT_H_

/*
 * Description of the video board, and the addresses
 * of its registers.
 */
struct VideoDevice {
	SVR_Regs		*sv_regs;
	DENC_Regs		*denc;
	struct video_info       *info;
	struct videoData        *data;
};
typedef struct VideoDevice	VDevice;

/*
 * Label and name of the device.
 */
struct video_info {
		/* device independent information */
	struct gfx_v_info	gfx_v_info;
		/* device dependent information */
	VDevice	*video_board;
	char name[GFX_V_INFO_NAME_SIZE];  /* unique name for this graphics */
	char label[GFX_V_INFO_LABEL_SIZE];/* manager's name for this board */
	unsigned int length;            /* size of a complete info struct */
					/* for this board */

};
typedef struct video_info	vInfo;

/*
 *
 */
struct videoData {
	vInfo	*info;
};
typedef struct videoData	vData;

#define	GFX_NAME_VIDEO_P0		"Starter Video 0"
#define	GFX_NAME_VIDEO_P1		"Starter Video 1"

/*
 * Video device structure.   XXX Some redundant data here.
 */
static struct video_board { 
	vInfo			*vb_info;
	struct video_open	*vb_owner;
	int			vb_number;   /* same as index in GfxBoards */
	struct queue		*vb_rq_ptr;
	struct queue		*vb_wq_ptr;
	/* gunk for talking to shimq  */
	short int		vb_devminor;	/* Identifies the shmiq */
	short int		vb_index;	/* Identifies the device */
	struct video_fncs	*vb_fncs;
	vData			*vb_data;
} ;
typedef struct video_board	VBoard;

/*
 *   Video driver structure.   There is one of these per open
 * board, per process.  This data structure was derived from
 * struct gfx_gfx, and so some fields ought to be deleted.
 */
struct video_open {

	/*  generic stuff  */
	VBoard			*vx_board;  /* pointer to board */

	struct proc		*vx_pp;     /* proc pointer */
	struct video_open	*vx_next;   /* ptr to next open gfx */

	/*  board driver stuff	*/
	struct video_fncs	*vx_fncs;
	vData			*vx_bdata;  /* ptr to driver private data */
	struct pregion		*vx_preg;
	unsigned long int	vx_flags;   /* flags needed by char side only */
	unsigned long int	vx_sflags;  /* flags needed by both sides */
	unsigned long int	vx_tlbmask; /* for late TLB flushing on (MP) */

	/*  rrm stuff  */
	void *vx_openrns;		/* RN's this gfx has open */
	void *vx_boundrn;		/* RN this gfx is currently bound to */

	/* share group fields */

	sema_t vx_sema;			/* semaphore for multiple sg procs */
};
typedef struct video_open	VideoOpen;

/*
 * Device independent interface to device dependent video
 * board functions.
 */

struct video_fncs {
			 /* functions for the Graphics Driver */
	int (*vx_Info)(vData *, void *, unsigned int);    
	int (*vx_Attach)(VideoOpen *, unsigned long int);
	int (*vx_Detach)(VideoOpen *);
	int (*vx_Initialize)(VideoOpen *);
	int (*vx_Start)(VideoOpen *);
	int (*vx_SchedRetraceEvent)(struct gfx_data *, struct rrm_rnode *);
	int (*vx_Setup)(VideoOpen *, caddr_t, int *);
	int (*vx_DimSetup)(VideoOpen *, caddr_t, int *, int);
	int (*vx_GetSetup)(VideoOpen *, caddr_t, int *);
	int (*vx_LoadMap)(VideoOpen *, caddr_t, int *, enum direction);
	int (*vx_GetMap)(VideoOpen *, caddr_t, int *, enum direction);
	int (*vx_SetParams)(VideoOpen *, caddr_t, int *);
	int (*vx_GetParams)(VideoOpen *, caddr_t, int *);
	int (*vx_GetFrame)(VideoOpen *, caddr_t, int *);
	int (*vx_PutFrame)(VideoOpen *, caddr_t, int *);
	int (*vx_Format)(VideoOpen *, caddr_t, int *, enum direction);
	int (*vx_GetStatus)(VideoOpen *, caddr_t, int *);

	/*
	 * routine to call graphics board driver specific functions.
	 */
	int (*vx_Dma)(VideoOpen *, caddr_t, int *);
	int (*vx_GetFeature)(VideoOpen *, caddr_t, int *);
	int (*vx_TuneBoard)(VideoOpen *, caddr_t, int *);
	int (*vx_Mmap)(int, vhandl_t *, int, int, int);
	int (*vx_UnMmap)(int, vhandl_t *);
	int (*vx_Poll)(int);

	/*
	 * provide a pointer to appropriate spl?() routines for
	 * the given hardware.
	 */
	int	(*vx_SplRetrace)();
};
typedef struct video_fncs	VFuncs;

#endif /* __SV1_STRUCT_H_ */
