/* #include "STAPUFT/gl/texparam.h" */

#ifndef _VGX_H_
#define _VGX_H_

#define VGX_FIFO_SIZE		0x4000	/* size of fifo hardware to map */

#if defined(IP17) || defined(IP17SKY)
#define VGX_TRIGGER_SIZE	0x2000	/* size of trigger hardware to map */
#else
#define VGX_TRIGGER_SIZE	0x1000	/* size of trigger hardware to map */
#endif
#define VGX_FEED_SIZE		0x3000	/* size of feedback area to map */
#define VGX_MAP_SIZE		(VGX_FIFO_SIZE + VGX_TRIGGER_SIZE + VGX_FEED_SIZE)

#define VGX_WID_CNT		16		/* number of display contexts in hw */
#define VGX_PCX_CNT		16		/* number of pipe contexts in hw */
#define VGX_PCX_SIZE		(32*1024)	/* maximum size of pipe context */
#define VGX_POLL_TIMEOUT	1000000		/* spin gm busy timeout */
#define VGX_FIFO_TIMEOUT	500		/* spin fifo full timeout */
#define VGX_GM_TIMEOUT		HZ		/* timed DMA timeout */
#define SPLVGX			spl5		/* spl to protect against intr */


/*
 * Structure returned by ioctl GETBOARDINFO when executed on a VGX.
 *
 * spanishness and alpha availability are unique to VGX hardware
 */
struct vgx_info {
	struct gfx_info gfx_info; 	/* device independent information */

	/* summary of VGX specific information */
	int	spans;			/* number of spans for head */
	int	flags;			/* flag indicating configuration of head */
#define VGX_INFO_ALPHA		0x00000001	/* alpha bit planes available */
#define VGX_INFO_EVMASK		0x0000000e	/* id from EV1 video car */
#define VGX_INFO_IMP5		0x00000010	/* IMP5, not IMP3 */
#define VGX_INFO_BIENDIAN	0x00000020	/* Bi-endian capable */
	/* detailed VGX specific information (used by GL) */
	gm_dispinfo_t	dispinfo;
};


#define VGX_BASE		10000
#define VGX_MAPPEDCOPY		(VGX_BASE + 1)
#define	VGX_ACCBUF		(VGX_BASE + 2)
#define	VGX_CALLTOGM		(VGX_BASE + 3)
#define VGX_RTNBUFBEGIN		(VGX_BASE + 4)
#define VGX_RTNBUFEND		(VGX_BASE + 5)
#define VGX_SETCURSORMODE	(VGX_BASE + 6)
#define VGX_SETCURSORCACHEID	(VGX_BASE + 7)
#define VGX_SETCURSORCOLORS	(VGX_BASE + 8)
#define VGX_MAPMEMORY		(VGX_BASE + 9)	/* for the IVL */
#define VGX_SETDISPLAYMODE	(VGX_BASE + 10)
#define VGX_VIDEO_PERIPH	(VGX_BASE + 11)	/* video peripheral command */
#define VGX_TEXTUREBIND		(VGX_BASE + 12)	/* texturebind */
#define VGX_VSYNC_SET		(VGX_BASE + 13) 
#define VGX_VSYNC_GET		(VGX_BASE + 14)
#define VGX_VSYNC_SLEEP		(VGX_BASE + 15)

/*
 *   VGX Ioctl args
 */

struct vgx_setobscured_args {
	int rnid;
	int obscuredflag;
};

struct vgx_setwinpos_args {
	int rnid;
	int xo,yo,xs,ys;
};

struct vgx_setwid_args {
	int rnid;
	int wid;
};

struct vgx_setdisplaymode_args {
   	long int wid;
	unsigned long int displaymode;
};

struct vgx_setcursormode_args {
	int	mode;
};

struct vgx_setcursorcacheid_args {
	int	cache_id;
};

struct vgx_setcursorcolors_args {
	int	colors[3];
};

struct vgx_video_periph_args {
	int	cmd;
	int	arg0;
	int	arg1;
	int	arg2;
	int	arg3;
};

/*
 * Structure to remember locked memory
 */
typedef struct vgx_lockmap_s {
	caddr_t			buffer;	/* user address */
	int			buflen;	/* length */
	struct vgx_lockmap_s	*next;	/* linked list pointer */
}  vgx_lockmap_t;


/* used by both mappedcopy and mapmemory */
struct vgx_mappedcopy_args {
	caddr_t buffer;		/* user virtual address of buffer */
	int	buflen;		/* length to transfer in bytes */
	int	flag;		/* 0 = load, 1 = store _to_ main memory */
	unsigned int cpcmd;	/* CP cmd */
	unsigned int cpdata[15];/* actual CP data values */
	int 	cpdatalen;	/* number of CP data items */
};

struct vgx_calltogm_args {
	/*
	 * NOTE: sendp actually points to struct gm_generic_cmd 
	 *	 recvp actually points to struct gm_generic_resp.
	 */
	void	*sendp;		/* Pointer to packet to send to GM	*/
	long	slen;		/* length in bytes of the packet	*/
	void	*recvp;		/* Ptr to recv packet be stuffed by GM  */
	long	rlen;		/* length in byte of receive packet	*/
};

/* These bit flags appear to set the flags field in vgx_rnode 
 * as well as (at least ) vgx_rtnbufbegin_args
 */
#define VGX_RN_TEXINDANGER	0x8	/* a texture is in accumulation buffer bank */
#define VGX_RN_ACCUMBUFFER	0x1	/* the rn uses the accumulation buffer */
#define	GM_RN_PICKING		0x2	/* pick/select in progress */
#define	GM_RN_FEEDBACK		0x4	/* feedback operation in progress */

struct vgx_rtnbufbegin_args {
	long flags;		/* pick or feedback? */
        char *buffer;           /* user buffer virtual address */
        long buflen;            /* length of buffer */
};

struct vgx_rtnbufend_args {
	unsigned int cpcmd;	/* CP cmd */
	unsigned int cpdata[2]; /* actual CP data values */
	int 	cpdatalen;	/* number of CP data items */
        long overflow;          /* buffer overflow flag */
};

struct vgx_texturebind_args {
	long	n;
	long	m;
	struct ulamslot *slots;
};

struct vgx_vsync_sleep_args {
	long	interval;
	long	mod;
};

/* arbitrary size chosen so we don't lock down too much mem */
#define VGX_MAX_RTNBUF		(128 * NBPC)	/* Return buf max size (bytes)*/

#define VGX_ACCBUFSTART		((1<<16)+(1<<15))

/* size of array used for allocations for structures describing textures */
#define NUM_TEXSHADOWS	1024

/*
 * State of a slot of graphics texture memory:
 * FREE     - memory does not contain a valid texture map.
 * UNLOADED - memory did contain a valid texture map and will likely 
	      need to again.
 * INACTIVE - memory contains a valid texture map for a valid process that is
 *            not currently running.
 * ACTIVE   - memory contains a valid texture map that may be used at any time.
 * RESERVED- transient state appears during kernel computations.
 */
#define FREE		0
#define UNLOADED	(FREE+1)
#define INACTIVE        (UNLOADED+1)
#define ACTIVE		(INACTIVE+1)
#define RESERVED	(ACTIVE+1)
#define ACCBUFUSE	(RESERVED+1)

#ifdef _KERNEL
/*
 * GM3 external status register (at base address provided in edtinit)
 *	status register is read only...
 *	to reset the board, write 1 then 0...
 */
#define	GM_SR_FIFO_NOT_HALF_FULL	0x01
#define	GM_SR_NOT_RESET			0x02
#define GM_SR_NOT_HALTED		0x04
#define	GM_CONTROLS_FIFO		0x08		/* GM3 only */
#define	GM_SR_DATAFIFO_NOT_HALF_FULL	0x10		/* GM3 only */
#define	GM_REVISION_MASK		0xe0		/* GM2, GM3 */
#define	GM_REVISION_2			0xa0		/* GM 2 */
#define	GM_REVISION_3			0x00		/* GM 3 */
#define	GM_REVISION_35			0x20		/* GM 3.5 */
#define GM_3WAYCXT_SHIFT		8
#define GM_3WAYCXT_MASK			(0xff << GM_3WAYCXT_SHIFT)

/*
 * GM3 3way count restore register resides at this offset from
 * the external status register.  it is a write only 32 bit register
 */
#define GM_3WAYCXT_OFFSET		0x200000

/*
 * the following structure is utilized to reference real
 * hardware registers.
 */
struct vgx_hw_reg {
	unsigned int reg;
};

/*
 * the following structure is utilized to access the
 * graphics pipe from the host.
 */
union vgx_pipe {
	float f;
	long i;
	unsigned long ul;
	long dummy[2];
};


/*
 * Structure to keep track of what textures are loaded in
 * the graphics texture memory.  It is also used for graphics
 * texture memory allocation.
 */
typedef struct texshadow {
	long  macr;		/* content of memory access control reg  */
	long  addr;		/* physical address in hw texture memory */
	long  state;		/* state of segment (see texparam.h) */
	long  size;		/* size of this segment (npixels rounded up to multiple of TEXPACKETSIZE*/
	unsigned long* vaddr;   /* User addr of the texture map (NULL for fogtable) */
	long  npixels;		/* Size of texture map in pixels */
	caddr_t uaddr;		/* User addr of the corresponding ulamslot structure */
	struct gfx_gfx *gfxp;	/* ptr to graph struct of owner if ACTIVE/INACTIVE */
	struct rrm_rnode *rnp;	/* Pointer to rendering node structure */
	struct texshadow *next;	/* pointer to next segment */
	struct texshadow *prev;	/* pointer to prev segment */
} texshadow_t;


/*
 * each instance of a VGX graphics head requires one of the following
 * structures to control the head.
 */
struct vgx_data {
	struct gfx_data		gfx_data;	/* device independent data */
	struct vgx_info		info;		/* vgx specific info */
	struct rrm_rnode	*rrn;		/* current rendering node */
	volatile struct vgx_hw_reg *gm_status;	/* K1 status addr for GM3 */
	volatile struct vgx_hw_reg *gm_3waycxt;	/* K1 3waycxt addr for GM3 */
	char			*gm_pipebase;	/* K1 pipe base addr */
	char			*trigbase;	/* K1 3-way base addr */
	host_to_gm_comm_t	*htogm;		/* host to gm comm area */
	gm_to_host_comm_t	*gmtoh;		/* gm to host area */
	void			*recvp;		/* Kernel space to receive GM data */
	int			recvlen;	/* Size of kernel space to receive GM data */
	volatile gm_shared_t	*gmshmem;	/* shared gm state info */
	int			gmslot;		/* slot ID for GM */
	lock_t			gmlock;		/* to lock the comm area */
	sema_t			gmcmd;		/* to semaphore the command interface */
	sema_t			gmdma;		/* to synchronise users doing GM DMA */
	sema_t			gmwaitdma;	/* semaphore to indicate DMA done */
	sema_t			gmwaitcmd;	/* a place to wait for GM commands */
	sema_t			gmwaitabt;	/* a place to wait for GM aborts */
	int			timiddma;	/* timeout ID for GM DMAs */
	int			timidabt;	/* timeout ID for GM ABORTs */
	uint	fifo_wait_mask;			/* what CPUs are spinning for fifo */
	int	timeoutcnt;			/* Number of times of fifo timeouts */
	int	swapwid[VGX_WID_CNT];		/* WID map for swapbuffers */
	uint			pcx_pt[16];	/* Phys addr to pass to GM for DMA of heavyweight cxt switch */
	char			*pcx_area;	/* Pointer to non-cached, non-paged area for GM DMA of context */
	int			gmdmalen;	/* bytes completed for last GM dma */
	struct rrm_rnode *loadedpcxs[VGX_PCX_CNT];
	unsigned long loadedmodes[VGX_WID_CNT];
	struct rrm_rnode *swaprnp[VGX_WID_CNT];
	struct rrm_rnode *retracelist;		/* waiting for vertical retrace list */
	long			retraceid;	/* unique retrace id generator */
	long			must_deactivate;

	/* Texture manager data structures below: */
	sema_t			texsema;	/* semaphore for serialising tex mgr code */

	long num_tex_seg; /* Number of texture segments in the system	*/
			/* For 5 spans, num_tex_seg = 1			*/
			/* For 10 spans, num_tex_seg =			*/
			/*	2  (accumulation buffer not available)	*/
			/*	3  (accumulation buffer available)	*/
			/* For SKYWR (Imp5) there is only one segment,  */
			/* because textures can cross bank boundaries,  */
			/* but its size varies from 1<<16 to 1<<16+1<<15*/
			/* to 1<<16+1<<15+1<<16 depending upon spanishness */
			/* and accbuffer availibility */

	long		acbuf_use_cnt;	/* Number of acbuf users on this pipe	*/
	texshadow_t	texshadow[NUM_TEXSHADOWS];
	texshadow_t*	freeptr;	/* Used for texture memory allocation 	*/
	texshadow_t*	texseg[3];	/* list of structures in segment. Indexed by segment number. */
	int cleanmask;
	int cur_vsync_cnt;		/* Current vsync count as given by the pipe */
	struct vgx_rnode *vsync_sleep_list;
};

/*
 * Device DEPENDENT rendering node structure
 */
struct vgx_rnode {
	unsigned long		gm3waycxt;

	texshadow_t*		texshadow[3];	/* pointers to texture shadow lists */
	long*			glaccbufcntptr;	/* pointer to shared memory in GL; zero on accbuf process death */

	struct RRM_ValidateClip validateclip;	/* squirreled-away data for loading during cxt sw (if rnid!=0) */

        /* user return buffer to copy out data at interrupt time */
	char			*rtnbufu;	/* user virtual address of buffer */
	char			*rtnbufk;	/* kernel virtual address of buffer */
	long			rtnbuflen;	/* buffer size (bytes) */
	long			rtnbufcount;	/* number bytes in buffer */
	long			rtnbufoverflow;	/* buffer overflowed (not a flag because we
						   change this at interrupt time, which could
						   get flags out of sync) */
	unsigned long		flags;		/* flags for picking,feedback,accumbuffer,texindanger */
	caddr_t			g_lockmap;	/* pointer to list of lockmaps (used by IVL) */

	struct rrm_rnode 	*retracenext;	/* waiting for vertical retrace next pointer */
	int			retraceid;	/* retrace id for this guy */
	int			cyclops;	/* this is a cyclops process */
	sema_t			vsync_sema;
	struct vgx_rnode 	*vsync_sleep_next;/* Points to next guy sleeping on a vsync */
	int			vsync_interval;	/* Number of vertical retraces before a wakeup */
	int			vsync_mod;	/* */
#ifdef IP17
	unsigned long 		ip17p3way_cntcmdreg; /* only 16 bits are valid */
	unsigned long		ip17p3way_ABCDregs[4];
#endif
};

#endif /* _KERNEL */


#endif /* _VGX_H_ */
